@@ -2,14 +2,30 @@
 %%% File    : cyrsasl_anonymous.erl
 %%% Author  : Magnus Henoch <henoch@dtek.chalmers.se>
 %%% Purpose : ANONYMOUS SASL mechanism
+%%%  See http://www.ietf.org/internet-drafts/draft-ietf-sasl-anon-05.txt
 %%% Created : 23 Aug 2005 by Magnus Henoch <henoch@dtek.chalmers.se>
-%%% Id      : $Id: cyrsasl_anonymous.erl 528 2006-04-07 00:51:53Z alexey $
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
-%% See http://www.ietf.org/internet-drafts/draft-ietf-sasl-anon-05.txt
-
 -module(cyrsasl_anonymous).
--vsn('$Revision: 528 $').
 
 -export([start/1, stop/0, mech_new/3, mech_step/2]).
 
@@ -35,5 +51,6 @@ mech_step(State, _ClientIn) ->
     %% Checks that the username is available
     case ejabberd_auth:is_user_exists(User, Server) of
 	true  -> {error, "not-authorized"};
-	false -> {ok, [{username, User}]}
+	false -> {ok, [{username, User},
+		       {auth_module, ejabberd_auth_anonymous}]}
     end.
