@@ -1,13 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : ejabberd_hooks.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Manage hooks
-%%% Created :  8 Aug 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_hooks.erl 370 2005-06-20 03:18:13Z alexey $
+%%% Created :  8 Aug 2004 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_hooks).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 -behaviour(gen_server).
 
@@ -98,7 +116,7 @@ init([]) ->
 %%          {stop, Reason, Reply, State}   | (terminate/2 is called)
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
-handle_call({add, Hook, Host, Module, Function, Seq}, From, State) ->
+handle_call({add, Hook, Host, Module, Function, Seq}, _From, State) ->
     Reply = case ets:lookup(hooks, {Hook, Host}) of
 		[{_, Ls}] ->
 		    El = {Seq, Module, Function},
@@ -116,7 +134,7 @@ handle_call({add, Hook, Host, Module, Function, Seq}, From, State) ->
 		    ok
 	    end,
     {reply, Reply, State};
-handle_call({delete, Hook, Host, Module, Function, Seq}, From, State) ->
+handle_call({delete, Hook, Host, Module, Function, Seq}, _From, State) ->
     Reply = case ets:lookup(hooks, {Hook, Host}) of
 		[{_, Ls}] ->
 		    NewLs = lists:delete({Seq, Module, Function}, Ls),
@@ -126,7 +144,7 @@ handle_call({delete, Hook, Host, Module, Function, Seq}, From, State) ->
 		    ok
 	    end,
     {reply, Reply, State};
-handle_call(Request, From, State) ->
+handle_call(_Request, _From, State) ->
     Reply = ok,
     {reply, Reply, State}.
 
@@ -136,7 +154,7 @@ handle_call(Request, From, State) ->
 %%          {noreply, State, Timeout} |
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
-handle_cast(Msg, State) ->
+handle_cast(_Msg, State) ->
     {noreply, State}.
 
 %%----------------------------------------------------------------------
@@ -145,7 +163,7 @@ handle_cast(Msg, State) ->
 %%          {noreply, State, Timeout} |
 %%          {stop, Reason, State}            (terminate/2 is called)
 %%----------------------------------------------------------------------
-handle_info(Info, State) ->
+handle_info(_Info, State) ->
     {noreply, State}.
 
 %%----------------------------------------------------------------------
@@ -153,7 +171,7 @@ handle_info(Info, State) ->
 %% Purpose: Shutdown the server
 %% Returns: any (ignored by gen_server)
 %%----------------------------------------------------------------------
-terminate(Reason, State) ->
+terminate(_Reason, _State) ->
     ok.
 
 
@@ -164,7 +182,7 @@ code_change(_OldVsn, State, _Extra) ->
 %%% Internal functions
 %%%----------------------------------------------------------------------
 
-run1([], Hook, Args) ->
+run1([], _Hook, _Args) ->
     ok;
 run1([{_Seq, Module, Function} | Ls], Hook, Args) ->
     case catch apply(Module, Function, Args) of
@@ -179,7 +197,7 @@ run1([{_Seq, Module, Function} | Ls], Hook, Args) ->
     end.
 
 
-run_fold1([], Hook, Val, Args) ->
+run_fold1([], _Hook, Val, _Args) ->
     Val;
 run_fold1([{_Seq, Module, Function} | Ls], Hook, Val, Args) ->
     case catch apply(Module, Function, [Val | Args]) of
