@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : ejabberd_sup.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : 
-%%% Created : 31 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_sup.erl 864 2007-08-09 14:24:33Z mremond $
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
+%%% Purpose : Erlang/OTP supervisor
+%%% Created : 31 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(ejabberd_sup).
--author('alexey@sevcom.net').
--vsn('$Revision: 864 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(supervisor).
 
@@ -17,7 +34,6 @@
 start_link() ->
     supervisor:start_link({local, ?MODULE}, ?MODULE, []).
 
-
 init([]) ->
     Hooks =
 	{ejabberd_hooks,
@@ -26,6 +42,20 @@ init([]) ->
 	 brutal_kill,
 	 worker,
 	 [ejabberd_hooks]},
+    NodeGroups =
+	{ejabberd_node_groups,
+	 {ejabberd_node_groups, start_link, []},
+	 permanent,
+	 brutal_kill,
+	 worker,
+	 [ejabberd_node_groups]},
+    SystemMonitor =
+	{ejabberd_system_monitor,
+	 {ejabberd_system_monitor, start_link, []},
+	 permanent,
+	 brutal_kill,
+	 worker,
+	 [ejabberd_system_monitor]},
     Router =
 	{ejabberd_router,
 	 {ejabberd_router, start_link, []},
@@ -116,6 +146,14 @@ init([]) ->
 	 infinity,
 	 supervisor,
 	 [ejabberd_tmp_sup]},
+    FrontendSocketSupervisor =
+	{ejabberd_frontend_socket_sup,
+	 {ejabberd_tmp_sup, start_link,
+	  [ejabberd_frontend_socket_sup, ejabberd_frontend_socket]},
+	 permanent,
+	 infinity,
+	 supervisor,
+	 [ejabberd_tmp_sup]},
     IQSupervisor =
 	{ejabberd_iq_sup,
 	 {ejabberd_tmp_sup, start_link,
@@ -126,6 +164,8 @@ init([]) ->
 	 [ejabberd_tmp_sup]},
     {ok, {{one_for_one, 10, 1},
 	  [Hooks,
+	   NodeGroups,
+	   SystemMonitor,
 	   Router,
 	   SM,
 	   S2S,
@@ -138,6 +178,7 @@ init([]) ->
 	   HTTPSupervisor,
 	   HTTPPollSupervisor,
 	   IQSupervisor,
+	   FrontendSocketSupervisor,
 	   Listener]}}.
 
 
