@@ -1,17 +1,37 @@
 %%%-------------------------------------------------------------------
 %%% File    : ejabberd_update.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : Update ejabberd
-%%% Created : 27 Jan 2006 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: ejabberd_c2s.erl 492 2006-01-25 00:35:12Z alexey $
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
+%%% Purpose : ejabberd code updater
+%%% Created : 27 Jan 2006 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%-------------------------------------------------------------------
 
 -module(ejabberd_update).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 %% API
 -export([update/0, update_info/0]).
 
+-include("ejabberd.hrl").
+
 %%====================================================================
 %% API
 %%====================================================================
@@ -22,7 +42,7 @@ update() ->
 		release_handler_1:eval_script(
 		  LowLevelScript, [],
 		  [{ejabberd, "", filename:join(Dir, "..")}]),
-	    io:format("eval: ~p~n", [Eval]),
+	    ?INFO_MSG("eval: ~p~n", [Eval]),
 	    Eval;
 	{error, Reason} ->
 	    {error, Reason}
@@ -37,28 +57,28 @@ update_info() ->
 	    UpdatedBeams =
 		lists:filter(
 		  fun(Module) ->
-			  {ok, {Module, [NewVsn]}} =
+			  {ok, {Module, NewVsn}} =
 			      beam_lib:version(code:which(Module)),
 			  case code:is_loaded(Module) of
 			      {file, _} ->
 				  Attrs = Module:module_info(attributes),
-				  {value, {vsn, [CurVsn]}} =
+				  {value, {vsn, CurVsn}} =
 				      lists:keysearch(vsn, 1, Attrs),
 				  NewVsn /= CurVsn;
 			      false ->
 				  false
 			  end
 		  end, Beams),
-	    io:format("beam files: ~p~n", [UpdatedBeams]),
+	    ?INFO_MSG("beam files: ~p~n", [UpdatedBeams]),
 	    Script = make_script(UpdatedBeams),
-	    io:format("script: ~p~n", [Script]),
+	    ?INFO_MSG("script: ~p~n", [Script]),
 	    LowLevelScript = make_low_level_script(UpdatedBeams, Script),
-	    io:format("low level script: ~p~n", [LowLevelScript]),
+	    ?INFO_MSG("low level script: ~p~n", [LowLevelScript]),
 	    Check =
 		release_handler_1:check_script(
 		  LowLevelScript,
 		  [{ejabberd, "", filename:join(Dir, "..")}]),
-	    io:format("check: ~p~n", [Check]),
+	    ?INFO_MSG("check: ~p~n", [Check]),
 	    {ok, Dir, UpdatedBeams, Script, LowLevelScript, Check};
 	{error, Reason} ->
 	    {error, Reason}
