@@ -5,7 +5,7 @@
 %%%           The interface is based on RFC 1823, and
 %%%           draft-ietf-asid-ldap-c-api-00.txt
 %%%
-%%% Copyright (C) 2000  Torbjrn Trnkvist, tnt@home.se
+%%% Copyright (C) 2000  Torbjorn Tornkvist, tnt@home.se
 %%% 
 %%% This program is free software; you can redistribute it and/or modify
 %%% it under the terms of the GNU General Public License as published by
@@ -32,8 +32,19 @@
 
 
 %%% Modified by Alexey Shchepin <alexey@sevcom.net>
+
+%%% Modified by Evgeniy Khramtsov <xram@jabber.ru>
+%%% Implemented queue for bind() requests to prevent pending binds.
+
+%%% Modified by Christophe Romain <christophe.romain@process-one.net>
+%%% Improve error case handling
+
+%%% Modified by Mickael Remond <mremond@process-one.net>
+%%% Now use ejabberd log mechanism
+
+
 %%% --------------------------------------------------------------------
--vc('$Id: eldap.erl 865 2007-08-09 14:55:53Z mremond $ ').
+-vc('$Id: eldap.erl 1237 2008-03-20 16:38:29Z mremond $ ').
 
 
 %%%----------------------------------------------------------------------
@@ -42,26 +53,26 @@
 %%%     connecting - actually disconnected, but retrying periodically
 %%%     wait_bind_response  - connected and sent bind request
 %%%     active - bound to LDAP Server and ready to handle commands
+%%%     active_bind - sent bind() request and waiting for response
 %%%----------------------------------------------------------------------
 
-%%-compile(export_all).
-%%-export([Function/Arity, ...]).
-
 -behaviour(gen_fsm).
 
+-include("ejabberd.hrl").
+
 %% External exports
--export([start_link/1, start_link/5, start_link/6]).
+-export([start_link/1, start_link/5]).
 
 -export([baseObject/0,singleLevel/0,wholeSubtree/0,close/1,
 	 equalityMatch/2,greaterOrEqual/2,lessOrEqual/2,
 	 approxMatch/2,search/2,substrings/2,present/1,
 	 'and'/1,'or'/1,'not'/1,modify/3, mod_add/2, mod_delete/2,
 	 mod_replace/2, add/3, delete/2, modify_dn/5, bind/3]).
--export([debug_level/2, get_status/1]).
+-export([get_status/1]).
 
 %% gen_fsm callbacks
 -export([init/1, connecting/2,
-	 connecting/3, wait_bind_response/3, active/3, handle_event/3,
+	 connecting/3, wait_bind_response/3, active/3, active_bind/3, handle_event/3,
 	 handle_sync_event/4, handle_info/3, terminate/3, code_change/4]).
 
 
@@ -71,25 +82,24 @@
 -include("eldap.hrl").
 
 -define(LDAP_VERSION, 3).
--define(RETRY_TIMEOUT, 5000).
+-define(RETRY_TIMEOUT, 500).
 -define(BIND_TIMEOUT, 10000).
--define(CMD_TIMEOUT, 5000).
+-define(CMD_TIMEOUT, 100000).
 -define(MAX_TRANSACTION_ID, 65535).
 -define(MIN_TRANSACTION_ID, 0).
 
 -record(eldap, {version = ?LDAP_VERSION,
-		hosts,	      % Possible hosts running LDAP servers
-		host = null,  % Connected Host LDAP server
-		port = 389 ,  % The LDAP server port
-		fd = null,    % Socket filedescriptor.
-		rootdn = "",  % Name of the entry to bind as
-		passwd,       % Password for (above) entry
-		id = 0,       % LDAP Request ID 
-		log,          % User provided log function
-		bind_timer,   % Ref to bind timeout
-		dict,         % dict holding operation params and results
-		debug_level   % Integer debug/logging level
-	       }).
+		hosts,         % Possible hosts running LDAP servers
+		host = null,   % Connected Host LDAP server
+		port = 389,    % The LDAP server port
+		fd = null,     % Socket filedescriptor.
+		rootdn = "",   % Name of the entry to bind as
+		passwd,        % Password for (above) entry
+		id = 0,        % LDAP Request ID 
+		bind_timer,    % Ref to bind timeout
+		dict,          % dict holding operation params and results
+		bind_q         % Queue for bind() requests
+	}).
 
 %%%----------------------------------------------------------------------
 %%% API
@@ -99,20 +109,8 @@ start_link(Name) ->
     gen_fsm:start_link({local, Reg_name}, ?MODULE, [], []).
 
 start_link(Name, Hosts, Port, Rootdn, Passwd) ->
-    Log = fun(N, Fmt, Args) -> io:format("---- " ++ Fmt, [Args]) end,
     Reg_name = list_to_atom("eldap_" ++ Name),
-    gen_fsm:start_link({local, Reg_name}, ?MODULE, {Hosts, Port, Rootdn, Passwd, Log}, []).
-
-start_link(Name, Hosts, Port, Rootdn, Passwd, Log) ->
-    Reg_name = list_to_atom("eldap_" ++ Name),
-    gen_fsm:start_link({local, Reg_name}, ?MODULE, {Hosts, Port, Rootdn, Passwd, Log}, []).
-
-%%% --------------------------------------------------------------------
-%%% Set Debug Level. 0 - none, 1 - errors, 2 - ldap events
-%%% --------------------------------------------------------------------
-debug_level(Handle, N) when integer(N) ->
-    Handle1 = get_handle(Handle),
-    gen_fsm:sync_send_all_state_event(Handle1, {debug_level,N}).
+    gen_fsm:start_link({local, Reg_name}, ?MODULE, {Hosts, Port, Rootdn, Passwd}, []).
 
 %%% --------------------------------------------------------------------
 %%% Get status of connection.
@@ -196,10 +194,10 @@ mod_replace(Type, Values) when list(Type), list(Values) -> m(replace, Type, Valu
 
 m(Operation, Type, Values) ->
     #'ModifyRequest_modification_SEQOF'{
-       operation = Operation,
-       modification = #'AttributeTypeAndValues'{
-	 type = Type,
-	 vals = Values}}.
+   operation = Operation,
+   modification = #'AttributeTypeAndValues'{
+     type = Type,
+     vals = Values}}.
 
 %%% --------------------------------------------------------------------
 %%% Modify an entry. Given an entry a number of modification
@@ -230,7 +228,7 @@ modify_dn(Handle, Entry, NewRDN, DelOldRDN, NewSup)
 bind(Handle, RootDN, Passwd) 
   when list(RootDN),list(Passwd) ->
     Handle1 = get_handle(Handle),
-    gen_fsm:sync_send_event(Handle1, {bind, RootDN, Passwd}).
+    gen_fsm:sync_send_event(Handle1, {bind, RootDN, Passwd}, infinity).
 
 %%% Sanity checks !
 
@@ -266,7 +264,7 @@ optional(Value) -> Value.
 %%% --------------------------------------------------------------------
 search(Handle, A) when record(A, eldap_search) ->
     call_search(Handle, A);
-search(Handle, L) when list(Handle), list(L) ->
+search(Handle, L) when list(L) ->
     case catch parse_search_args(L) of
 	{error, Emsg}                  -> {error, Emsg};
 	{'EXIT', Emsg}                 -> {error, Emsg};
@@ -275,11 +273,11 @@ search(Handle, L) when list(Handle), list(L) ->
 
 call_search(Handle, A) ->
     Handle1 = get_handle(Handle),
-    gen_fsm:sync_send_event(Handle1, {search, A}).
+    gen_fsm:sync_send_event(Handle1, {search, A}, infinity).
 
 parse_search_args(Args) ->
     parse_search_args(Args, #eldap_search{scope = wholeSubtree}).
-		       
+
 parse_search_args([{base, Base}|T],A) ->
     parse_search_args(T,A#eldap_search{base = Base});
 parse_search_args([{filter, Filter}|T],A) ->
@@ -292,7 +290,9 @@ parse_search_args([{types_only, TypesOnly}|T],A) ->
     parse_search_args(T,A#eldap_search{types_only = TypesOnly});
 parse_search_args([{timeout, Timeout}|T],A) when integer(Timeout) ->
     parse_search_args(T,A#eldap_search{timeout = Timeout});
-parse_search_args([H|T],A) ->
+parse_search_args([{limit, Limit}|T],A) when is_integer(Limit) ->
+    parse_search_args(T,A#eldap_search{limit = Limit});
+parse_search_args([H|_],_) ->
     throw({error,{unknown_arg, H}});
 parse_search_args([],A) ->
     A.
@@ -370,20 +370,19 @@ get_handle(Name) when list(Name) -> list_to_atom("eldap_" ++ Name).
 %%----------------------------------------------------------------------
 init([]) ->
     case get_config() of
-	{ok, Hosts, Rootdn, Passwd, Log} ->
-	    init({Hosts, Rootdn, Passwd, Log});
+	{ok, Hosts, Rootdn, Passwd} ->
+	    init({Hosts, Rootdn, Passwd});
 	{error, Reason} ->
 	    {stop, Reason}
     end;
-init({Hosts, Port, Rootdn, Passwd, Log}) ->
+init({Hosts, Port, Rootdn, Passwd}) ->
     {ok, connecting, #eldap{hosts = Hosts,
 			    port = Port,
 			    rootdn = Rootdn,
 			    passwd = Passwd,
 			    id = 0,
-			    log = Log,
 			    dict = dict:new(),
-			    debug_level = 0}, 0}.
+			    bind_q = queue:new()}, 0}.
 
 %%----------------------------------------------------------------------
 %% Func: StateName/2
@@ -406,24 +405,39 @@ connecting(timeout, S) ->
 %%          {stop, Reason, NewStateData}                          |
 %%          {stop, Reason, Reply, NewStateData}                    
 %%----------------------------------------------------------------------
-connecting(Event, From, S) ->
+connecting(_Event, _From, S) ->
     Reply = {error, connecting},
     {reply, Reply, connecting, S}.
 
-wait_bind_response(Event, From, S) ->
+wait_bind_response(_Event, _From, S) ->
     Reply = {error, wait_bind_response},
     {reply, Reply, wait_bind_response, S}.
 
 active(Event, From, S) ->
     case catch send_command(Event, From, S) of
 	{ok, NewS} ->
-	    {next_state, active, NewS};
+	    case Event of
+		{bind, _, _} ->
+		    {next_state, active_bind, NewS};
+		_ ->
+		    {next_state, active, NewS}
+	    end;
 	{error, Reason} ->
 	    {reply, {error, Reason}, active, S};
 	{'EXIT', Reason} ->
 	    {reply, {error, Reason}, active, S}
     end.
 
+active_bind({bind, RootDN, Passwd}, From, #eldap{bind_q=Q} = S) ->
+    NewQ = queue:in({{bind, RootDN, Passwd}, From}, Q),
+    {next_state, active_bind, S#eldap{bind_q=NewQ}};
+active_bind(Event, From, S) ->
+    case catch send_command(Event, From, S) of
+	{ok, NewS}       -> {next_state, active_bind, NewS};
+	{error, Reason}  -> {reply, {error, Reason}, active_bind, S};
+	{'EXIT', Reason} -> {reply, {error, Reason}, active_bind, S}
+    end.
+
 %%----------------------------------------------------------------------
 %% Func: handle_event/3
 %% Called when gen_fsm:send_all_state_event/2 is invoked.
@@ -431,11 +445,24 @@ active(Event, From, S) ->
 %%          {next_state, NextStateName, NextStateData, Timeout} |
 %%          {stop, Reason, NewStateData}                         
 %%----------------------------------------------------------------------
-handle_event(close, StateName, S) ->
+handle_event(close, _StateName, S) ->
     gen_tcp:close(S#eldap.fd),
     {stop, closed, S};
 
-handle_event(Event, StateName, S) ->
+handle_event(process_bind_q, active_bind, #eldap{bind_q=Q} = S) ->
+    case queue:out(Q) of
+	{{value, {BindEvent, To}}, NewQ} ->
+	    NewStateData = case catch send_command(BindEvent, To, S) of
+			       {ok, NewS}       -> NewS;
+			       {error, Reason}  -> gen_fsm:reply(To, {error, Reason}), S;
+			       {'EXIT', Reason} -> gen_fsm:reply(To, {error, Reason}), S
+			   end,
+	    {next_state, active_bind, NewStateData#eldap{bind_q=NewQ}};
+	{empty, Q} ->
+	    {next_state, active, S}
+    end;
+
+handle_event(_Event, StateName, S) ->
     {next_state, StateName, S}.
 
 %%----------------------------------------------------------------------
@@ -448,15 +475,8 @@ handle_event(Event, StateName, S) ->
 %%          {stop, Reason, NewStateData}                          |
 %%          {stop, Reason, Reply, NewStateData}                    
 %%----------------------------------------------------------------------
-handle_sync_event({debug_level, N}, From, StateName, S) ->
-    {reply, ok, StateName, S#eldap{debug_level = N}};
-
-handle_sync_event(Event, From, StateName, S) ->
-    {reply, {StateName, S}, StateName, S};
-
-handle_sync_event(Event, From, StateName, S) ->
-    Reply = ok,
-    {reply, Reply, StateName, S}.
+handle_sync_event(_Event, _From, StateName, S) ->
+    {reply, {StateName, S}, StateName, S}.
 
 %%----------------------------------------------------------------------
 %% Func: handle_info/3
@@ -468,44 +488,52 @@ handle_sync_event(Event, From, StateName, S) ->
 %%
 %% Packets arriving in various states
 %%
-handle_info({tcp, Socket, Data}, connecting, S) ->
-    log1("eldap. tcp packet received when disconnected!~n~p~n", [Data], S),
+handle_info({tcp, _Socket, Data}, connecting, S) ->
+    ?DEBUG("eldap. tcp packet received when disconnected!~n~p", [Data]),
     {next_state, connecting, S};
 
-handle_info({tcp, Socket, Data}, wait_bind_response, S) ->
+handle_info({tcp, _Socket, Data}, wait_bind_response, S) ->
     cancel_timer(S#eldap.bind_timer),
     case catch recvd_wait_bind_response(Data, S) of
 	bound                -> {next_state, active, S};
-	{fail_bind, Reason}  -> close_and_retry(S),
+	{fail_bind, _Reason}  -> close_and_retry(S),
 				{next_state, connecting, S#eldap{fd = null}};
-	{'EXIT', Reason}     -> close_and_retry(S),
+	{'EXIT', _Reason}     -> close_and_retry(S),
 				{next_state, connecting, S#eldap{fd = null}};
-	{error, Reason}      -> close_and_retry(S),
+	{error, _Reason}      -> close_and_retry(S),
 				{next_state, connecting, S#eldap{fd = null}}
     end;
 
-handle_info({tcp, Socket, Data}, active, S) ->
+handle_info({tcp, _Socket, Data}, StateName, S)
+  when StateName==active; StateName==active_bind ->
     case catch recvd_packet(Data, S) of
 	{reply, Reply, To, NewS} -> gen_fsm:reply(To, Reply),
-				    {next_state, active, NewS};
-	{ok, NewS}               -> {next_state, active, NewS};
-	{'EXIT', Reason}         -> {next_state, active, S};
-	{error, Reason}          -> {next_state, active, S}
+				    {next_state, StateName, NewS};
+	{ok, NewS}               -> {next_state, StateName, NewS};
+	{'EXIT', _Reason}         -> {next_state, StateName, S};
+	{error, _Reason}          -> {next_state, StateName, S}
     end;
 
-handle_info({tcp_closed, Socket}, All_fsm_states, S) ->
-    F = fun(Id, [{Timer, From, Name}|Res]) ->
+handle_info({tcp_closed, _Socket}, Fsm_state, S) ->
+    ?WARNING_MSG("LDAP server closed the connection: ~s:~p~nIn State: ~p",
+	  [S#eldap.host, S#eldap.port ,Fsm_state]),
+    F = fun(_Id, [{Timer, From, _Name}|_]) ->
 		gen_fsm:reply(From, {error, tcp_closed}),
 		cancel_timer(Timer)
 	end,
     dict:map(F, S#eldap.dict),
-    retry_connect(),
-    {next_state, connecting, S#eldap{fd = null,
-				     dict = dict:new()}};
+    {ok, NextState, NewS} = connect_bind(S#eldap{fd = null,
+						 dict = dict:new(),
+						 bind_q=queue:new()}),
+    {next_state, NextState, NewS};
 
-handle_info({tcp_error, Socket, Reason}, Fsm_state, S) ->
-    log1("eldap received tcp_error: ~p~nIn State: ~p~n", [Reason, Fsm_state], S),
+handle_info({tcp_error, _Socket, Reason}, Fsm_state, S) ->
+    ?DEBUG("eldap received tcp_error: ~p~nIn State: ~p", [Reason, Fsm_state]),
+    %% XXX wouldn't it be safer to try reconnect ?
+    %% if we were waiting a result, we may mait forever
+    %% cause request is probably lost....
     {next_state, Fsm_state, S};
+
 %%
 %% Timers
 %%
@@ -513,14 +541,14 @@ handle_info({timeout, Timer, {cmd_timeout, Id}}, active, S) ->
     case cmd_timeout(Timer, Id, S) of
 	{reply, To, Reason, NewS} -> gen_fsm:reply(To, Reason),
 				     {next_state, active, NewS};
-	{error, Reason}           -> {next_state, active, S}
+	{error, _Reason}           -> {next_state, active, S}
     end;
 
 handle_info({timeout, retry_connect}, connecting, S) ->
     {ok, NextState, NewS} = connect_bind(S), 
     {next_state, NextState, NewS};
 
-handle_info({timeout, Timer, bind_timeout}, wait_bind_response, S) ->
+handle_info({timeout, _Timer, bind_timeout}, wait_bind_response, S) ->
     close_and_retry(S),
     {next_state, connecting, S#eldap{fd = null}};
 
@@ -528,8 +556,8 @@ handle_info({timeout, Timer, bind_timeout}, wait_bind_response, S) ->
 %% Make sure we don't fill the message queue with rubbish
 %%
 handle_info(Info, StateName, S) ->
-    log1("eldap. Unexpected Info: ~p~nIn state: ~p~n when StateData is: ~p~n",
-			[Info, StateName, S], S),
+    ?DEBUG("eldap. Unexpected Info: ~p~nIn state: ~p~n when StateData is: ~p",
+	 [Info, StateName, S]),
     {next_state, StateName, S}.
 
 %%----------------------------------------------------------------------
@@ -537,7 +565,7 @@ handle_info(Info, StateName, S) ->
 %% Purpose: Shutdown the fsm
 %% Returns: any
 %%----------------------------------------------------------------------
-terminate(Reason, StateName, StatData) ->
+terminate(_Reason, _StateName, _StatData) ->
     ok.
 
 %%----------------------------------------------------------------------
@@ -545,7 +573,7 @@ terminate(Reason, StateName, StatData) ->
 %% Purpose: Convert process state when code is changed
 %% Returns: {ok, NewState, NewStateData}
 %%----------------------------------------------------------------------
-code_change(OldVsn, StateName, S, Extra) ->
+code_change(_OldVsn, StateName, S, _Extra) ->
     {ok, StateName, S}.
 
 %%%----------------------------------------------------------------------
@@ -556,20 +584,23 @@ send_command(Command, From, S) ->
     {Name, Request} = gen_req(Command),
     Message = #'LDAPMessage'{messageID  = Id,
 			     protocolOp = {Name, Request}},
-    log2("~p~n",[{Name, Request}], S),
+    ?DEBUG("~p~n",[{Name, Request}]),
     {ok, Bytes} = asn1rt:encode('ELDAPv3', 'LDAPMessage', Message),
-    ok = gen_tcp:send(S#eldap.fd, Bytes),
-    Timer = erlang:start_timer(?CMD_TIMEOUT, self(), {cmd_timeout, Id}),
-    New_dict = dict:store(Id, [{Timer, From, Name}], S#eldap.dict),
-    {ok, S#eldap{id = Id,
-		 dict = New_dict}}.
+    case gen_tcp:send(S#eldap.fd, Bytes) of
+    ok ->
+	Timer = erlang:start_timer(?CMD_TIMEOUT, self(), {cmd_timeout, Id}),
+	New_dict = dict:store(Id, [{Timer, From, Name}], S#eldap.dict),
+	{ok, S#eldap{id = Id, dict = New_dict}};
+    Error ->
+	Error
+    end.
 
 gen_req({search, A}) ->
     {searchRequest,
      #'SearchRequest'{baseObject   = A#eldap_search.base,
 		      scope        = v_scope(A#eldap_search.scope),
 		      derefAliases = neverDerefAliases,
-		      sizeLimit    = 0, % no size limit
+		      sizeLimit    = A#eldap_search.limit,
 		      timeLimit    = v_timeout(A#eldap_search.timeout),
 		      typesOnly    = v_bool(A#eldap_search.types_only),
 		      filter       = v_filter(A#eldap_search.filter),
@@ -614,29 +645,30 @@ recvd_packet(Pkt, S) ->
     case asn1rt:decode('ELDAPv3', 'LDAPMessage', Pkt) of
 	{ok,Msg} ->
 	    Op = Msg#'LDAPMessage'.protocolOp,
-	    log2("~p~n",[Op], S),
+	    ?DEBUG("~p",[Op]),
 	    Dict = S#eldap.dict,
 	    Id = Msg#'LDAPMessage'.messageID,
 	    {Timer, From, Name, Result_so_far} = get_op_rec(Id, Dict),
 	    case {Name, Op} of
 		{searchRequest, {searchResEntry, R}} when
-		      record(R,'SearchResultEntry') ->
+		record(R,'SearchResultEntry') ->
 		    New_dict = dict:append(Id, R, Dict),
 		    {ok, S#eldap{dict = New_dict}};
 		{searchRequest, {searchResDone, Result}} ->
-		    case Result#'LDAPResult'.resultCode of
-			success ->
+		    Reason = Result#'LDAPResult'.resultCode,
+		    if
+			Reason==success; Reason=='sizeLimitExceeded' ->
 			    {Res, Ref} = polish(Result_so_far),
 			    New_dict = dict:erase(Id, Dict),
 			    cancel_timer(Timer),
 			    {reply, #eldap_search_result{entries = Res,
 							 referrals = Ref}, From,
-			                              S#eldap{dict = New_dict}};
-			Reason ->
+			     S#eldap{dict = New_dict}};
+			true ->
 			    New_dict = dict:erase(Id, Dict),
 			    cancel_timer(Timer),
 			    {reply, {error, Reason}, From, S#eldap{dict = New_dict}}
-			end;
+		    end;
 		{searchRequest, {searchResRef, R}} ->
 		    New_dict = dict:append(Id, R, Dict),
 		    {ok, S#eldap{dict = New_dict}};
@@ -664,28 +696,29 @@ recvd_packet(Pkt, S) ->
 		    New_dict = dict:erase(Id, Dict),
 		    cancel_timer(Timer),
 		    Reply = check_bind_reply(Result, From),
+		    gen_fsm:send_all_state_event(self(), process_bind_q),
 		    {reply, Reply, From, S#eldap{dict = New_dict}};
 		{OtherName, OtherResult} ->
 		    New_dict = dict:erase(Id, Dict),
 		    cancel_timer(Timer),
 		    {reply, {error, {invalid_result, OtherName, OtherResult}},
-		            From, S#eldap{dict = New_dict}}
+		     From, S#eldap{dict = New_dict}}
 	    end;
 	Error -> Error
     end.
 
-check_reply(#'LDAPResult'{resultCode = success}, From) ->
+check_reply(#'LDAPResult'{resultCode = success}, _From) ->
     ok;
-check_reply(#'LDAPResult'{resultCode = Reason}, From) ->
+check_reply(#'LDAPResult'{resultCode = Reason}, _From) ->
     {error, Reason};
-check_reply(Other, From) ->
+check_reply(Other, _From) ->
     {error, Other}.
 
-check_bind_reply(#'BindResponse'{resultCode = success}, From) ->
+check_bind_reply(#'BindResponse'{resultCode = success}, _From) ->
     ok;
-check_bind_reply(#'BindResponse'{resultCode = Reason}, From) ->
+check_bind_reply(#'BindResponse'{resultCode = Reason}, _From) ->
     {error, Reason};
-check_bind_reply(Other, From) ->
+check_bind_reply(Other, _From) ->
     {error, Other}.
 
 get_op_rec(Id, Dict) ->
@@ -709,7 +742,7 @@ recvd_wait_bind_response(Pkt, S) ->
     check_tag(Pkt),
     case asn1rt:decode('ELDAPv3', 'LDAPMessage', Pkt) of
 	{ok,Msg} ->
-	    log2("~p", [Msg], S),
+	    ?DEBUG("~p", [Msg]),
 	    check_id(S#eldap.id, Msg#'LDAPMessage'.messageID),
 	    case Msg#'LDAPMessage'.protocolOp of
 		{bindResponse, Result} ->
@@ -742,9 +775,9 @@ cancel_timer(Timer) ->
 %%% Sanity check of received packet
 check_tag(Data) ->
     case asn1rt_ber_bin:decode_tag(Data) of
-	{Tag, Data1, Rb} ->
+	{_Tag, Data1, _Rb} ->
 	    case asn1rt_ber_bin:decode_length(Data1) of
-		{{Len,Data2}, Rb2} -> ok;
+		{{_Len,_Data2}, _Rb2} -> ok;
 		_ -> throw({error,decoded_tag_length})
 	    end;
 	_ -> throw({error,decoded_tag})
@@ -773,8 +806,8 @@ cmd_timeout(Timer, Id, S) ->
 		    {reply, From, {timeout,
 				   #eldap_search_result{entries = Res1,
 							referrals = Ref1}},
-		                   S#eldap{dict = New_dict}};
-		Others ->
+		     S#eldap{dict = New_dict}};
+		_ ->
 		    New_dict = dict:erase(Id, Dict),
 		    {reply, From, {error, timeout}, S#eldap{dict = New_dict}}
 	    end;
@@ -809,6 +842,7 @@ polish([], Res, Ref) ->
 connect_bind(S) ->
     Host = next_host(S#eldap.host, S#eldap.hosts),
     TcpOpts = [{packet, asn1}, {active, true}, {keepalive, true}, binary],
+    ?INFO_MSG("LDAP connection on ~s:~p", [Host, S#eldap.port]),
     case gen_tcp:connect(Host, S#eldap.port, TcpOpts) of
 	{ok, Socket} ->
 	    case bind_request(Socket, S) of
@@ -819,14 +853,14 @@ connect_bind(S) ->
 							host = Host,
 							bind_timer = Timer}};
 		{error, Reason} ->
+		    ?ERROR_MSG("LDAP bind failed on ~s:~p~nReason: ~p", [Host, S#eldap.port, Reason]),
 		    gen_tcp:close(Socket),
-		    erlang:send_after(?RETRY_TIMEOUT, self(),
-				      {timeout, retry_connect}),
+		    retry_connect(),
 		    {ok, connecting, S#eldap{host = Host}}
 	    end;
 	{error, Reason} ->
-	    erlang:send_after(?RETRY_TIMEOUT, self(), 
-			      {timeout, retry_connect}),
+	    ?ERROR_MSG("LDAP connection failed on ~s:~p~nReason: ~p", [Host, S#eldap.port, Reason]),
+	    retry_connect(),
 	    {ok, connecting, S#eldap{host = Host}}
     end.
 
@@ -837,10 +871,12 @@ bind_request(Socket, S) ->
 			 authentication = {simple, S#eldap.passwd}},
     Message = #'LDAPMessage'{messageID  = Id,
 			     protocolOp = {bindRequest, Req}},
-    log2("Message:~p~n",[Message], S),
+    ?DEBUG("Bind Request Message:~p~n",[Message]),
     {ok, Bytes} = asn1rt:encode('ELDAPv3', 'LDAPMessage', Message),
-    ok = gen_tcp:send(Socket, Bytes),
-    {ok, S#eldap{id = Id}}.
+    case gen_tcp:send(Socket, Bytes) of
+	ok -> {ok, S#eldap{id = Id}};
+	Error -> Error
+    end.
 
 %% Given last tried Server, find next one to try
 next_host(null, [H|_]) -> H;			% First time, take first
@@ -848,9 +884,9 @@ next_host(Host, Hosts) ->			% Find next in turn
     next_host(Host, Hosts, Hosts).
 
 next_host(Host, [Host], Hosts) -> hd(Hosts);	% Wrap back to first
-next_host(Host, [Host|Tail], Hosts) -> hd(Tail);	% Take next
-next_host(Host, [], Hosts) -> hd(Hosts);	% Never connected before? (shouldn't happen)
-next_host(Host, [H|T], Hosts) -> next_host(Host, T, Hosts).
+next_host(Host, [Host|Tail], _Hosts) -> hd(Tail);	% Take next
+next_host(_Host, [], Hosts) -> hd(Hosts);	% Never connected before? (shouldn't happen)
+next_host(Host, [_|T], Hosts) -> next_host(Host, T, Hosts).
 
 
 %%% --------------------------------------------------------------------
@@ -879,7 +915,7 @@ v_modifications(Mods) ->
 
 v_substr([{Key,Str}|T]) when list(Str),Key==initial;Key==any;Key==final ->
     [{Key,Str}|v_substr(T)];
-v_substr([H|T]) ->
+v_substr([H|_]) ->
     throw({error,{substring_arg,H}});
 v_substr([]) -> 
     [].
@@ -911,8 +947,8 @@ get_config() ->
     case file:consult(File) of
 	{ok, Entries} ->
 	    case catch parse(Entries) of
-		{ok, Hosts, Port, Rootdn, Passwd, Log} ->
-		    {ok, Hosts, Port, Rootdn, Passwd, Log};
+		{ok, Hosts, Port, Rootdn, Passwd} ->
+		    {ok, Hosts, Port, Rootdn, Passwd};
 		{error, Reason} ->
 		    {error, Reason};
 		{'EXIT', Reason} ->
@@ -927,14 +963,13 @@ parse(Entries) ->
      get_hosts(host, Entries),
      get_integer(port, Entries),
      get_list(rootdn, Entries),
-     get_list(passwd, Entries),
-     get_log(log, Entries)}.
+     get_list(passwd, Entries)}.
 
 get_integer(Key, List) ->
     case lists:keysearch(Key, 1, List) of
 	{value, {Key, Value}} when integer(Value) ->
 	    Value;
-	{value, {Key, Value}} ->
+	{value, {Key, _Value}} ->
 	    throw({error, "Bad Value in Config for " ++ atom_to_list(Key)});
 	false ->
 	    throw({error, "No Entry in Config for " ++ atom_to_list(Key)})
@@ -944,22 +979,12 @@ get_list(Key, List) ->
     case lists:keysearch(Key, 1, List) of
 	{value, {Key, Value}} when list(Value) ->
 	    Value;
-	{value, {Key, Value}} ->
+	{value, {Key, _Value}} ->
 	    throw({error, "Bad Value in Config for " ++ atom_to_list(Key)});
 	false ->
 	    throw({error, "No Entry in Config for " ++ atom_to_list(Key)})
     end.
 
-get_log(Key, List) ->
-    case lists:keysearch(Key, 1, List) of
-	{value, {Key, Value}} when function(Value) ->
-	    Value;
-	{value, {Key, Else}} ->
-	    false;
-	false ->
-	    fun(Level, Format, Args) -> io:format("--- " ++ Format, Args) end
-    end.
-
 get_hosts(Key, List) ->
     lists:map(fun({Key1, {A,B,C,D}}) when integer(A),
 					  integer(B),
@@ -970,7 +995,7 @@ get_hosts(Key, List) ->
 		 ({Key1, Value}) when list(Value),
 				      Key == Key1->
 		      Value;
-		 ({Else, Value}) ->
+		 ({_Else, _Value}) ->
 		      throw({error, "Bad Hostname in config"}) 
 	      end, List).
 
@@ -981,15 +1006,3 @@ bump_id(#eldap{id = Id}) when Id > ?MAX_TRANSACTION_ID ->
     ?MIN_TRANSACTION_ID;
 bump_id(#eldap{id = Id}) ->
     Id + 1.
-
-%%% --------------------------------------------------------------------
-%%% Log routines. Call a user provided log routine Fun.
-%%% --------------------------------------------------------------------
-
-log1(Str, Args, #eldap{log = Fun, debug_level = N}) -> log(Fun, Str, Args, 1, N).
-log2(Str, Args, #eldap{log = Fun, debug_level = N}) -> log(Fun, Str, Args, 2, N).
-
-log(Fun, Str, Args, This_level, Status) when function(Fun), This_level =< Status ->
-    catch Fun(This_level, Str, Args);
-log(_, _, _, _, _) -> 
-    ok.
