@@ -3,7 +3,25 @@
 %%% Author  : Leif Johansson <leifj@it.su.se>
 %%% Purpose : External authentication using a simple port-driver
 %%% Created : 30 Jul 2004 by Leif Johansson <leifj@it.su.se>
-%%% Id      : $Id: extauth.erl 374 2005-07-13 03:24:13Z alexey $
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(extauth).
@@ -12,6 +30,7 @@
 -export([start/2, stop/1, init/2,
 	 check_password/3, set_password/3, is_user_exists/2]).
 
+-include("ejabberd.hrl").
 
 start(Host, ExtPrg) ->
     spawn(?MODULE, init, [Host, ExtPrg]).
@@ -58,7 +77,7 @@ loop(Port) ->
 		    exit(normal)
 	    end;
 	{'EXIT', Port, Reason} ->
-	    io:format("~p ~n", [Reason]),
+	    ?CRITICAL_MSG("~p ~n", [Reason]),
 	    exit(port_terminated)
     end.
 
