@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : jd2ejd.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Import of jabberd1.4 user spool file
-%%% Created :  2 Feb 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: jd2ejd.erl 507 2006-02-15 04:15:54Z alexey $
+%%% Created :  2 Feb 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(jd2ejd).
--author('alexey@sevcom.net').
--vsn('$Revision: 507 $ ').
+-author('alexey@process-one.net').
 
 %% External exports
 -export([import_file/1,
@@ -38,7 +55,7 @@ import_file(File) ->
 				    ?ERROR_MSG(
 				       "Error while processing file \"~s\": ~p~n",
 				       [File, Reason]),
-				       {error, Reason};
+				    {error, Reason};
 				_ ->
 				    ok
 			    end;
@@ -95,7 +112,7 @@ process_xdb(User, Server, {xmlelement, Name, _Attrs, Els}) ->
     end.
 
 
-xdb_data(User, Server, {xmlcdata, _CData}) ->
+xdb_data(_User, _Server, {xmlcdata, _CData}) ->
     ok;
 xdb_data(User, Server, {xmlelement, _Name, Attrs, _Els} = El) ->
     From = jlib:make_jid(User, Server, ""),
