@@ -1,13 +1,34 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_announce.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Manage announce messages
-%%% Created : 11 Aug 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_announce.erl 486 2006-01-19 02:17:31Z alexey $
+%%% Created : 11 Aug 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
+%%% Implements a small subset of XEP-0133: Service Administration
+%%% Version 1.1 (2005-08-19)
+
 -module(mod_announce).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 -behaviour(gen_mod).
 
@@ -31,6 +52,9 @@
 
 -define(PROCNAME, ejabberd_announce).
 
+-define(NS_ADMINL(Sub), ["http:","jabber.org","protocol","admin", Sub]).
+tokenize(Node) -> string:tokens(Node, "/#").
+
 start(Host, _Opts) ->
     mnesia:create_table(motd, [{disc_copies, [node()]},
 			       {attributes, record_info(fields, motd)}]),
@@ -57,6 +81,9 @@ loop() ->
 	{announce_all, From, To, Packet} ->
 	    announce_all(From, To, Packet),
 	    loop();
+	{announce_all_hosts_all, From, To, Packet} ->
+	    announce_all_hosts_all(From, To, Packet),
+	    loop();
 	{announce_online, From, To, Packet} ->
 	    announce_online(From, To, Packet),
 	    loop();
@@ -66,12 +93,21 @@ loop() ->
 	{announce_motd, From, To, Packet} ->
 	    announce_motd(From, To, Packet),
 	    loop();
+	{announce_all_hosts_motd, From, To, Packet} ->
+	    announce_all_hosts_motd(From, To, Packet),
+	    loop();
 	{announce_motd_update, From, To, Packet} ->
 	    announce_motd_update(From, To, Packet),
 	    loop();
+	{announce_all_hosts_motd_update, From, To, Packet} ->
+	    announce_all_hosts_motd_update(From, To, Packet),
+	    loop();
 	{announce_motd_delete, From, To, Packet} ->
 	    announce_motd_delete(From, To, Packet),
 	    loop();
+	{announce_all_hosts_motd_delete, From, To, Packet} ->
+	    announce_all_hosts_motd_delete(From, To, Packet),
+	    loop();
 	_ ->
 	    loop()
     end.
@@ -84,13 +120,13 @@ stop(Host) ->
     ejabberd_hooks:delete(disco_local_items, Host, ?MODULE, disco_items, 50),
     ejabberd_hooks:delete(local_send_to_resource_hook, Host,
 			  ?MODULE, announce, 50),
-    ejabberd_hooks:delete(sm_register_connection_hook, Host,
+    ejabberd_hooks:delete(user_available_hook, Host,
 			  ?MODULE, send_motd, 50),
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     exit(whereis(Proc), stop),
     {wait, Proc}.
 
-% Announcing via messages to a custom resource
+%% Announcing via messages to a custom resource
 announce(From, To, Packet) ->
     case To of
 	#jid{luser = "", lresource = Res} ->
@@ -100,6 +136,9 @@ announce(From, To, Packet) ->
 		{"announce/all", "message"} ->
 		    Proc ! {announce_all, From, To, Packet},
 		    stop;
+		{"announce/all-hosts/all", "message"} ->
+		    Proc ! {announce_all_hosts_all, From, To, Packet},
+		    stop;
 		{"announce/online", "message"} ->
 		    Proc ! {announce_online, From, To, Packet},
 		    stop;
@@ -109,12 +148,21 @@ announce(From, To, Packet) ->
 		{"announce/motd", "message"} ->
 		    Proc ! {announce_motd, From, To, Packet},
 		    stop;
+		{"announce/all-hosts/motd", "message"} ->
+		    Proc ! {announce_all_hosts_motd, From, To, Packet},
+		    stop;
 		{"announce/motd/update", "message"} ->
 		    Proc ! {announce_motd_update, From, To, Packet},
 		    stop;
+		{"announce/all-hosts/motd/update", "message"} ->
+		    Proc ! {announce_all_hosts_motd_update, From, To, Packet},
+		    stop;
 		{"announce/motd/delete", "message"} ->
 		    Proc ! {announce_motd_delete, From, To, Packet},
 		    stop;
+		{"announce/all-hosts/motd/delete", "message"} ->
+		    Proc ! {announce_all_hosts_motd_delete, From, To, Packet},
+		    stop;
 		_ ->
 		    ok
 	    end;
@@ -122,8 +170,8 @@ announce(From, To, Packet) ->
 	    ok
     end.
 
-%-------------------------------------------------------------------------
-% Announcing via ad-hoc commands
+%%-------------------------------------------------------------------------
+%% Announcing via ad-hoc commands
 -define(INFO_COMMAND(Lang, Node),
         [{xmlelement, "identity",
 	  [{"category", "automation"},
@@ -131,32 +179,41 @@ announce(From, To, Packet) ->
 	   {"name", get_title(Lang, Node)}], []}]).
 
 disco_identity(Acc, _From, _To, Node, Lang) ->
-    case Node of
-	"announce/all" ->
+    LNode = tokenize(Node),
+    case LNode of
+	?NS_ADMINL("announce") ->
 	    ?INFO_COMMAND(Lang, Node);
-	"announce/all-hosts/online" ->
+	?NS_ADMINL("announce-allhosts") ->
 	    ?INFO_COMMAND(Lang, Node);
-	"announce/online" ->
+	?NS_ADMINL("announce-all") ->
 	    ?INFO_COMMAND(Lang, Node);
-	"announce/motd" ->
+	?NS_ADMINL("announce-all-allhosts") ->
 	    ?INFO_COMMAND(Lang, Node);
-	"announce/motd/delete" ->
+	?NS_ADMINL("set-motd") ->
 	    ?INFO_COMMAND(Lang, Node);
-	"announce/motd/update" ->
+	?NS_ADMINL("set-motd-allhosts") ->
+	    ?INFO_COMMAND(Lang, Node);
+	?NS_ADMINL("edit-motd") ->
+	    ?INFO_COMMAND(Lang, Node);
+	?NS_ADMINL("edit-motd-allhosts") ->
+	    ?INFO_COMMAND(Lang, Node);
+	?NS_ADMINL("delete-motd") ->
+	    ?INFO_COMMAND(Lang, Node);
+	?NS_ADMINL("delete-motd-allhosts") ->
 	    ?INFO_COMMAND(Lang, Node);
 	_ ->
 	    Acc
     end.
 
-%-------------------------------------------------------------------------
+%%-------------------------------------------------------------------------
 
 -define(INFO_RESULT(Allow, Feats),
-    case Allow of
-	deny ->
-	    {error, ?ERR_FORBIDDEN};
-	allow ->
-	    {result, Feats}
-    end).
+	case Allow of
+	    deny ->
+		{error, ?ERR_FORBIDDEN};
+	    allow ->
+		{result, Feats}
+	end).
 
 disco_features(Acc, From, #jid{lserver = LServer} = _To,
 	       "announce", _Lang) ->
@@ -175,17 +232,6 @@ disco_features(Acc, From, #jid{lserver = LServer} = _To,
 	    end
     end;
 
-disco_features(Acc, From, #jid{lserver = LServer} = _To,
-	       "announce/all-hosts/online", _Lang) ->
-    case gen_mod:is_loaded(LServer, mod_adhoc) of
-	false ->
-	    Acc;
-	_ ->
-	    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
-	    Allow = acl:match_rule(global, Access, From),
-	    ?INFO_RESULT(Allow, [?NS_COMMANDS])
-    end;
-
 disco_features(Acc, From, #jid{lserver = LServer} = _To,
 	       Node, _Lang) ->
     case gen_mod:is_loaded(LServer, mod_adhoc) of
@@ -194,23 +240,35 @@ disco_features(Acc, From, #jid{lserver = LServer} = _To,
 	_ ->
 	    Access = gen_mod:get_module_opt(LServer, ?MODULE, access, none),
 	    Allow = acl:match_rule(LServer, Access, From),
+	    AccessGlobal = gen_mod:get_module_opt(global, ?MODULE, access, none),
+	    AllowGlobal = acl:match_rule(global, AccessGlobal, From),
 	    case Node of
-		"announce/all" ->
+		?NS_ADMIN ++ "#announce" ->
 		    ?INFO_RESULT(Allow, [?NS_COMMANDS]);
-		"announce/online" ->
+		?NS_ADMIN ++ "#announce-all" ->
 		    ?INFO_RESULT(Allow, [?NS_COMMANDS]);
-		"announce/motd" ->
+		?NS_ADMIN ++ "#set-motd" ->
 		    ?INFO_RESULT(Allow, [?NS_COMMANDS]);
-		"announce/motd/delete" ->
+		?NS_ADMIN ++ "#edit-motd" ->
 		    ?INFO_RESULT(Allow, [?NS_COMMANDS]);
-		"announce/motd/update" ->
+		?NS_ADMIN ++ "#delete-motd" ->
 		    ?INFO_RESULT(Allow, [?NS_COMMANDS]);
+		?NS_ADMIN ++ "#announce-allhosts" ->
+		    ?INFO_RESULT(AllowGlobal, [?NS_COMMANDS]);
+		?NS_ADMIN ++ "#announce-all-allhosts" ->
+		    ?INFO_RESULT(AllowGlobal, [?NS_COMMANDS]);
+		?NS_ADMIN ++ "#set-motd-allhosts" ->
+		    ?INFO_RESULT(AllowGlobal, [?NS_COMMANDS]);
+		?NS_ADMIN ++ "#edit-motd-allhosts" ->
+		    ?INFO_RESULT(AllowGlobal, [?NS_COMMANDS]);
+		?NS_ADMIN ++ "#delete-motd-allhosts" ->
+		    ?INFO_RESULT(AllowGlobal, [?NS_COMMANDS]);
 		_ ->
 		    Acc
 	    end
     end.
 
-%-------------------------------------------------------------------------
+%%-------------------------------------------------------------------------
 
 -define(NODE_TO_ITEM(Lang, Server, Node),
 	{xmlelement, "item",
@@ -220,12 +278,12 @@ disco_features(Acc, From, #jid{lserver = LServer} = _To,
 	 []}).
 
 -define(ITEMS_RESULT(Allow, Items),
-    case Allow of
-	deny ->
-	    {error, ?ERR_FORBIDDEN};
-	allow ->
-	    {result, Items}
-    end).
+	case Allow of
+	    deny ->
+		{error, ?ERR_FORBIDDEN};
+	    allow ->
+		{result, Items}
+	end).
 
 disco_items(Acc, From, #jid{lserver = LServer, server = Server} = _To,
 	    "", Lang) ->
@@ -257,17 +315,6 @@ disco_items(Acc, From, #jid{lserver = LServer} = To, "announce", Lang) ->
 	    announce_items(Acc, From, To, Lang)
     end;
 
-disco_items(Acc, From, #jid{lserver = LServer} = _To,
-	    "announce/all-hosts/online", _Lang) ->
-    case gen_mod:is_loaded(LServer, mod_adhoc) of
-	false ->
-	    Acc;
-	_ ->
-	    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
-	    Allow = acl:match_rule(global, Access, From),
-	    ?ITEMS_RESULT(Allow, [])
-    end;
-
 disco_items(Acc, From, #jid{lserver = LServer} = _To, Node, _Lang) ->
     case gen_mod:is_loaded(LServer, mod_adhoc) of
 	false ->
@@ -275,40 +322,56 @@ disco_items(Acc, From, #jid{lserver = LServer} = _To, Node, _Lang) ->
 	_ ->
 	    Access = gen_mod:get_module_opt(LServer, ?MODULE, access, none),
 	    Allow = acl:match_rule(LServer, Access, From),
+	    AccessGlobal = gen_mod:get_module_opt(global, ?MODULE, access, none),
+	    AllowGlobal = acl:match_rule(global, AccessGlobal, From),
 	    case Node of
-		"announce/all" ->
+		?NS_ADMIN ++ "#announce" ->
 		    ?ITEMS_RESULT(Allow, []);
-		"announce/online" ->
+		?NS_ADMIN ++ "#announce-all" ->
 		    ?ITEMS_RESULT(Allow, []);
-		"announce/motd" ->
+		?NS_ADMIN ++ "#set-motd" ->
 		    ?ITEMS_RESULT(Allow, []);
-		"announce/motd/delete" ->
+		?NS_ADMIN ++ "#edit-motd" ->
 		    ?ITEMS_RESULT(Allow, []);
-		"announce/motd/update" ->
+		?NS_ADMIN ++ "#delete-motd" ->
 		    ?ITEMS_RESULT(Allow, []);
+		?NS_ADMIN ++ "#announce-allhosts" ->
+		    ?ITEMS_RESULT(AllowGlobal, []);
+		?NS_ADMIN ++ "#announce-all-allhosts" ->
+		    ?ITEMS_RESULT(AllowGlobal, []);
+		?NS_ADMIN ++ "#set-motd-allhosts" ->
+		    ?ITEMS_RESULT(AllowGlobal, []);
+		?NS_ADMIN ++ "#edit-motd-allhosts" ->
+		    ?ITEMS_RESULT(AllowGlobal, []);
+		?NS_ADMIN ++ "#delete-motd-allhosts" ->
+		    ?ITEMS_RESULT(AllowGlobal, []);
 		_ ->
 		    Acc
 	    end
     end.
 
-%-------------------------------------------------------------------------
+%%-------------------------------------------------------------------------
 
 announce_items(Acc, From, #jid{lserver = LServer, server = Server} = _To, Lang) ->
     Access1 = gen_mod:get_module_opt(LServer, ?MODULE, access, none),
     Nodes1 = case acl:match_rule(LServer, Access1, From) of
 		 allow ->
-		     [?NODE_TO_ITEM(Lang, Server, "announce/all"),
-		      ?NODE_TO_ITEM(Lang, Server, "announce/online"),
-		      ?NODE_TO_ITEM(Lang, Server, "announce/motd"),
-		      ?NODE_TO_ITEM(Lang, Server, "announce/motd/delete"),
-		      ?NODE_TO_ITEM(Lang, Server, "announce/motd/update")];
+		     [?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#announce"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#announce-all"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#set-motd"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#edit-motd"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#delete-motd")];
 		 deny ->
 		     []
 	     end,
     Access2 = gen_mod:get_module_opt(global, ?MODULE, access, none),
     Nodes2 = case acl:match_rule(global, Access2, From) of
 		 allow ->
-		     [?NODE_TO_ITEM(Lang, Server, "announce/all-hosts/online")];
+		     [?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#announce-allhosts"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#announce-all-allhosts"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#set-motd-allhosts"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#edit-motd-allhosts"),
+		      ?NODE_TO_ITEM(Lang, Server, ?NS_ADMIN ++ "#delete-motd-allhosts")];
 		 deny ->
 		     []
 	     end,
@@ -323,43 +386,55 @@ announce_items(Acc, From, #jid{lserver = LServer, server = Server} = _To, Lang)
 	    {result, Items ++ Nodes1 ++ Nodes2}
     end.
 
-%-------------------------------------------------------------------------
+%%-------------------------------------------------------------------------
 
--define(COMMANDS_RESULT(Allow, From, To, Request),
+commands_result(Allow, From, To, Request) ->
     case Allow of
 	deny ->
 	    {error, ?ERR_FORBIDDEN};
 	allow ->
 	    announce_commands(From, To, Request)
-    end).
-
-announce_commands(_Acc, From, To,
-		  #adhoc_request{
-		     node = "announce/all-hosts/online"} = Request) ->
-    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
-    Allow = acl:match_rule(global, Access, From),
-    ?COMMANDS_RESULT(Allow, From, To, Request);
-
-announce_commands(Acc, From, #jid{lserver = LServer} = To,
-		  #adhoc_request{node = Node} = Request) ->
-    Access = gen_mod:get_module_opt(LServer, ?MODULE, access, none),
-    Allow = acl:match_rule(LServer, Access, From),
-    case Node of
-	"announce/all" ->
-	    ?COMMANDS_RESULT(Allow, From, To, Request);
-	"announce/online" ->
-	    ?COMMANDS_RESULT(Allow, From, To, Request);
-	"announce/motd" ->
-	    ?COMMANDS_RESULT(Allow, From, To, Request);
-	"announce/motd/delete" ->
-	    ?COMMANDS_RESULT(Allow, From, To, Request);
-	"announce/motd/update" ->
-	    ?COMMANDS_RESULT(Allow, From, To, Request);
-	_ ->
-	    Acc
     end.
 
-%-------------------------------------------------------------------------
+
+announce_commands(Acc, From, #jid{lserver = LServer} = To,
+		  #adhoc_request{ node = Node} = Request) ->
+    LNode = tokenize(Node),
+    F = fun() ->
+		Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
+		Allow = acl:match_rule(global, Access, From),
+		commands_result(Allow, From, To, Request)
+	end,
+    R = case LNode of
+	    ?NS_ADMINL("announce-allhosts") -> F();
+	    ?NS_ADMINL("announce-all-allhosts") -> F();
+	    ?NS_ADMINL("set-motd-allhosts") -> F();
+	    ?NS_ADMINL("edit-motd-allhosts") -> F();
+	    ?NS_ADMINL("delete-motd-allhosts") -> F();
+	    _ ->
+		Access = gen_mod:get_module_opt(LServer, ?MODULE, access, none),
+		Allow = acl:match_rule(LServer, Access, From),
+		case LNode of
+		    ?NS_ADMINL("announce") ->
+			commands_result(Allow, From, To, Request);
+		    ?NS_ADMINL("announce-all") ->
+			commands_result(Allow, From, To, Request);
+		    ?NS_ADMINL("set-motd") ->
+			commands_result(Allow, From, To, Request);
+		    ?NS_ADMINL("edit-motd") ->
+			commands_result(Allow, From, To, Request);
+		    ?NS_ADMINL("delete-motd") ->
+			commands_result(Allow, From, To, Request);
+		    _ ->
+			unknown
+		end
+	end,
+    case R of
+	unknown -> Acc;
+	_ -> {stop, R}
+    end.
+
+%%-------------------------------------------------------------------------
 
 announce_commands(From, To,
 		  #adhoc_request{lang = Lang,
@@ -378,10 +453,11 @@ announce_commands(From, To,
 				   #adhoc_response{status = canceled});
        XData == false, ActionIsExecute ->
 	    %% User requests form
+	    Elements = generate_adhoc_form(Lang, Node, To#jid.lserver),
 	    adhoc:produce_response(
 	      Request,
 	      #adhoc_response{status = executing,
-			      elements = [generate_adhoc_form(Lang, Node)]});
+			      elements = [Elements]});
        XData /= false, ActionIsExecute ->
 	    %% User returns form.
 	    case jlib:parse_xdata_submit(XData) of
@@ -394,32 +470,54 @@ announce_commands(From, To,
 	    {error, ?ERR_BAD_REQUEST}
     end.
 
-generate_adhoc_form(Lang, Node) ->
+-define(VVALUE(Val),
+	{xmlelement, "value", [], [{xmlcdata, Val}]}).
+-define(VVALUEL(Val),
+	case Val of
+	    "" -> [];
+	    _ -> [?VVALUE(Val)]
+	end).
+-define(TVFIELD(Type, Var, Val),
+	{xmlelement, "field", [{"type", Type},
+			       {"var", Var}],
+	 ?VVALUEL(Val)}).
+-define(HFIELD(), ?TVFIELD("hidden", "FORM_TYPE", ?NS_ADMIN)).
+
+generate_adhoc_form(Lang, Node, ServerHost) ->
+    LNode = tokenize(Node),
+    {OldSubject, OldBody} = if (LNode == ?NS_ADMINL("edit-motd")) 
+			       or (LNode == ?NS_ADMINL("edit-motd-allhosts")) ->
+				    get_stored_motd(ServerHost);
+			       true -> 
+				    {[], []}
+			    end,
     {xmlelement, "x",
      [{"xmlns", ?NS_XDATA},
       {"type", "form"}],
-     [{xmlelement, "title", [], [{xmlcdata, get_title(Lang, Node)}]}]
+     [?HFIELD(),
+      {xmlelement, "title", [], [{xmlcdata, get_title(Lang, Node)}]}]
      ++
-      if Node == "announce/motd/delete" ->
-	      [{xmlelement, "field",
+     if (LNode == ?NS_ADMINL("delete-motd"))
+	or (LNode == ?NS_ADMINL("delete-motd-allhosts")) ->
+	     [{xmlelement, "field",
 	       [{"var", "confirm"},
 		{"type", "boolean"},
 		{"label", translate:translate(Lang, "Really delete message of the day?")}],
 	       [{xmlelement, "value",
-		[],
-		[{xmlcdata, "true"}]}]}];
-	 true ->
-	      [{xmlelement, "field", 
-		[{"var", "subject"},
-		 {"type", "text-single"},
-		 {"label", translate:translate(Lang, "Subject")}],
-		[]},
-	       {xmlelement, "field",
-		[{"var", "body"},
-		 {"type", "text-multi"},
-		 {"label", translate:translate(Lang, "Message body")}],
-		[]}]
-      end}.
+		 [],
+		 [{xmlcdata, "true"}]}]}];
+	true ->
+	     [{xmlelement, "field", 
+	       [{"var", "subject"},
+		{"type", "text-single"},
+		{"label", translate:translate(Lang, "Subject")}],
+	       ?VVALUEL(OldSubject)},
+	      {xmlelement, "field",
+	       [{"var", "body"},
+		{"type", "text-multi"},
+		{"label", translate:translate(Lang, "Message body")}],
+	       ?VVALUEL(OldBody)}]
+     end}.
 
 join_lines([]) ->
     [];
@@ -478,37 +576,53 @@ handle_adhoc_form(From, #jid{lserver = LServer} = To,
 
     Proc = gen_mod:get_module_proc(LServer, ?PROCNAME),
     case {Node, Body} of
-	{"announce/motd/delete", _} ->
+	{?NS_ADMIN ++ "#delete-motd", _} ->
 	    if	Confirm ->
 		    Proc ! {announce_motd_delete, From, To, Packet},
 		    adhoc:produce_response(Response);
-	       true ->
+		true ->
+		    adhoc:produce_response(Response)
+	    end;
+	{?NS_ADMIN ++ "#delete-motd-allhosts", _} ->
+	    if	Confirm ->
+		    Proc ! {announce_all_hosts_motd_delete, From, To, Packet},
+		    adhoc:produce_response(Response);
+		true ->
 		    adhoc:produce_response(Response)
 	    end;
 	{_, []} ->
 	    %% An announce message with no body is definitely an operator error.
 	    %% Throw an error and give him/her a chance to send message again.
 	    {error, ?ERRT_NOT_ACCEPTABLE(
-		      Lang,
-		      "No body provided for announce message")};
+		       Lang,
+		       "No body provided for announce message")};
 	%% Now send the packet to ?PROCNAME.
 	%% We don't use direct announce_* functions because it
 	%% leads to large delay in response and <iq/> queries processing
-	{"announce/all", _} ->
-	    Proc ! {announce_all, From, To, Packet},
-	    adhoc:produce_response(Response);
-	{"announce/online", _} ->
+	{?NS_ADMIN ++ "#announce", _} ->
 	    Proc ! {announce_online, From, To, Packet},
 	    adhoc:produce_response(Response);
-	{"announce/all-hosts/online", _} ->	    
+	{?NS_ADMIN ++ "#announce-allhosts", _} ->	    
 	    Proc ! {announce_all_hosts_online, From, To, Packet},
 	    adhoc:produce_response(Response);
-	{"announce/motd", _} ->
+	{?NS_ADMIN ++ "#announce-all", _} ->
+	    Proc ! {announce_all, From, To, Packet},
+	    adhoc:produce_response(Response);
+	{?NS_ADMIN ++ "#announce-all-allhosts", _} ->	    
+	    Proc ! {announce_all_hosts_all, From, To, Packet},
+	    adhoc:produce_response(Response);
+	{?NS_ADMIN ++ "#set-motd", _} ->
 	    Proc ! {announce_motd, From, To, Packet},
 	    adhoc:produce_response(Response);
-	{"announce/motd/update", _} ->
+	{?NS_ADMIN ++ "#set-motd-allhosts", _} ->	    
+	    Proc ! {announce_all_hosts_motd, From, To, Packet},
+	    adhoc:produce_response(Response);
+	{?NS_ADMIN ++ "#edit-motd", _} ->
 	    Proc ! {announce_motd_update, From, To, Packet},
 	    adhoc:produce_response(Response);
+	{?NS_ADMIN ++ "#edit-motd-allhosts", _} ->	    
+	    Proc ! {announce_all_hosts_motd_update, From, To, Packet},
+	    adhoc:produce_response(Response);
 	_ ->
 	    %% This can't happen, as we haven't registered any other
 	    %% command nodes.
@@ -517,20 +631,28 @@ handle_adhoc_form(From, #jid{lserver = LServer} = To,
 
 get_title(Lang, "announce") ->
     translate:translate(Lang, "Announcements");
-get_title(Lang, "announce/all") ->
+get_title(Lang, ?NS_ADMIN ++ "#announce-all") ->
     translate:translate(Lang, "Send announcement to all users");
-get_title(Lang, "announce/online") ->
+get_title(Lang, ?NS_ADMIN ++ "#announce-all-allhosts") ->
+    translate:translate(Lang, "Send announcement to all users on all hosts");
+get_title(Lang, ?NS_ADMIN ++ "#announce") ->
     translate:translate(Lang, "Send announcement to all online users");
-get_title(Lang, "announce/all-hosts/online") ->
+get_title(Lang, ?NS_ADMIN ++ "#announce-allhosts") ->
     translate:translate(Lang, "Send announcement to all online users on all hosts");
-get_title(Lang, "announce/motd") ->
+get_title(Lang, ?NS_ADMIN ++ "#set-motd") ->
     translate:translate(Lang, "Set message of the day and send to online users");
-get_title(Lang, "announce/motd/update") ->
+get_title(Lang, ?NS_ADMIN ++ "#set-motd-allhosts") ->
+    translate:translate(Lang, "Set message of the day on all hosts and send to online users");
+get_title(Lang, ?NS_ADMIN ++ "#edit-motd") ->
     translate:translate(Lang, "Update message of the day (don't send)");
-get_title(Lang, "announce/motd/delete") ->
-    translate:translate(Lang, "Delete message of the day").
+get_title(Lang, ?NS_ADMIN ++ "#edit-motd-allhosts") ->
+    translate:translate(Lang, "Update message of the day on all hosts (don't send)");
+get_title(Lang, ?NS_ADMIN ++ "#delete-motd") ->
+    translate:translate(Lang, "Delete message of the day");
+get_title(Lang, ?NS_ADMIN ++ "#delete-motd-allhosts") ->
+    translate:translate(Lang, "Delete message of the day on all hosts").
 
-%-------------------------------------------------------------------------
+%%-------------------------------------------------------------------------
 
 announce_all(From, To, Packet) ->
     Host = To#jid.lserver,
@@ -542,10 +664,25 @@ announce_all(From, To, Packet) ->
 	allow ->
 	    Local = jlib:make_jid("", To#jid.server, ""),
 	    lists:foreach(
-		fun({User, Server}) ->
-			Dest = jlib:make_jid(User, Server, ""),
-			ejabberd_router:route(Local, Dest, Packet)
-		end, ejabberd_auth:get_vh_registered_users(Host))
+	      fun({User, Server}) ->
+		      Dest = jlib:make_jid(User, Server, ""),
+		      ejabberd_router:route(Local, Dest, Packet)
+	      end, ejabberd_auth:get_vh_registered_users(Host))
+    end.
+
+announce_all_hosts_all(From, To, Packet) ->
+    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
+    case acl:match_rule(global, Access, From) of
+	deny ->
+	    Err = jlib:make_error_reply(Packet, ?ERR_FORBIDDEN),
+	    ejabberd_router:route(To, From, Err);
+	allow ->
+	    Local = jlib:make_jid("", To#jid.server, ""),
+	    lists:foreach(
+	      fun({User, Server}) ->
+		      Dest = jlib:make_jid(User, Server, ""),
+		      ejabberd_router:route(Local, Dest, Packet)
+	      end, ejabberd_auth:dirty_get_registered_users())
     end.
 
 announce_online(From, To, Packet) ->
@@ -589,18 +726,32 @@ announce_motd(From, To, Packet) ->
 	    Err = jlib:make_error_reply(Packet, ?ERR_FORBIDDEN),
 	    ejabberd_router:route(To, From, Err);
 	allow ->
-	    announce_motd_update(To#jid.lserver, Packet),
-	    Sessions = ejabberd_sm:get_vh_session_list(Host),
-	    announce_online1(Sessions, To#jid.server, Packet),
-	    F = fun() ->
-			lists:foreach(
-			  fun({U, S, _R}) ->
-				  mnesia:write(#motd_users{us = {U, S}})
-			  end, Sessions)
-		end,
-	    mnesia:transaction(F)
+	    announce_motd(Host, Packet)
     end.
 
+announce_all_hosts_motd(From, To, Packet) ->
+    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
+    case acl:match_rule(global, Access, From) of
+	deny ->
+	    Err = jlib:make_error_reply(Packet, ?ERR_FORBIDDEN),
+	    ejabberd_router:route(To, From, Err);
+	allow ->
+	    Hosts = ?MYHOSTS,
+	    [announce_motd(Host, Packet) || Host <- Hosts]
+    end.
+
+announce_motd(Host, Packet) ->
+    announce_motd_update(Host, Packet),
+    Sessions = ejabberd_sm:get_vh_session_list(Host),
+    announce_online1(Sessions, Host, Packet),
+    F = fun() ->
+		lists:foreach(
+		  fun({U, S, _R}) ->
+			  mnesia:write(#motd_users{us = {U, S}})
+		  end, Sessions)
+	end,
+    mnesia:transaction(F).
+
 announce_motd_update(From, To, Packet) ->
     Host = To#jid.lserver,
     Access = gen_mod:get_module_opt(Host, ?MODULE, access, none),
@@ -612,6 +763,17 @@ announce_motd_update(From, To, Packet) ->
 	    announce_motd_update(Host, Packet)
     end.
 
+announce_all_hosts_motd_update(From, To, Packet) ->
+    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
+    case acl:match_rule(global, Access, From) of
+	deny ->
+	    Err = jlib:make_error_reply(Packet, ?ERR_FORBIDDEN),
+	    ejabberd_router:route(To, From, Err);
+	allow ->
+	    Hosts = ?MYHOSTS,
+	    [announce_motd_update(Host, Packet) || Host <- Hosts]
+    end.
+
 announce_motd_update(LServer, Packet) ->
     announce_motd_delete(LServer),
     F = fun() ->
@@ -630,6 +792,17 @@ announce_motd_delete(From, To, Packet) ->
 	    announce_motd_delete(Host)
     end.
 
+announce_all_hosts_motd_delete(From, To, Packet) ->
+    Access = gen_mod:get_module_opt(global, ?MODULE, access, none),
+    case acl:match_rule(global, Access, From) of
+	deny ->
+	    Err = jlib:make_error_reply(Packet, ?ERR_FORBIDDEN),
+	    ejabberd_router:route(To, From, Err);
+	allow ->
+	    Hosts = ?MYHOSTS,
+	    [announce_motd_delete(Host) || Host <- Hosts]
+    end.
+
 announce_motd_delete(LServer) ->
     F = fun() ->
 		mnesia:delete({motd, LServer}),
@@ -664,7 +837,16 @@ send_motd(#jid{luser = LUser, lserver = LServer} = JID) ->
 	    ok
     end.
 
-%-------------------------------------------------------------------------
+get_stored_motd(LServer) ->
+    case catch mnesia:dirty_read({motd, LServer}) of
+	[#motd{packet = Packet}] ->
+	    {xml:get_subtag_cdata(Packet, "subject"),
+	     xml:get_subtag_cdata(Packet, "body")};
+	_ ->
+	    {"", ""}
+    end.
+
+%%-------------------------------------------------------------------------
 
 update_tables() ->
     update_motd_table(),
@@ -754,4 +936,3 @@ update_motd_users_table() ->
 	    ?INFO_MSG("Recreating motd_users table", []),
 	    mnesia:transform_table(motd_users, ignore, Fields)
     end.
-
