@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_disco.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Service Discovery (JEP-0030) support
-%%% Created :  1 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_disco.erl 486 2006-01-19 02:17:31Z alexey $
+%%% Created :  1 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_disco).
--author('alexey@sevcom.net').
--vsn('$Revision: 486 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_mod).
 
@@ -168,12 +185,12 @@ process_local_iq_info(From, To, #iq{type = Type, lang = Lang,
 	    end
     end.
 
-get_local_identity(_Acc, _From, _To, [], _Lang) ->
-     [{xmlelement, "identity",
-       [{"category", "server"},
-	{"type", "im"},
-	{"name", "ejabberd"}], []}];
- 
+get_local_identity(Acc, _From, _To, [], _Lang) ->
+    Acc ++ [{xmlelement, "identity",
+	     [{"category", "server"},
+	      {"type", "im"},
+	      {"name", "ejabberd"}], []}];
+
 get_local_identity(Acc, _From, _To, _Node, _Lang) ->
     Acc.
 
