@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_echo.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : 
-%%% Created : 15 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_echo.erl 496 2006-02-02 05:00:27Z alexey $
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
+%%% Purpose : Simple ejabberd module.
+%%% Created : 15 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_echo).
--author('alexey@sevcom.net').
--vsn('$Revision: 496 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_server).
 -behaviour(gen_mod).
@@ -52,7 +69,9 @@ start(Host, Opts) ->
 stop(Host) ->
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     gen_server:call(Proc, stop),
-    supervisor:stop_child(ejabberd_sup, Proc).
+    supervisor:terminate_child(ejabberd_sup, Proc),
+    supervisor:delete_child(ejabberd_sup, Proc).
+
 
 %%====================================================================
 %% gen_server callbacks
@@ -66,7 +85,7 @@ stop(Host) ->
 %% Description: Initiates the server
 %%--------------------------------------------------------------------
 init([Host, Opts]) ->
-    MyHost = gen_mod:get_opt(host, Opts, "echo." ++ Host),
+    MyHost = gen_mod:get_opt_host(Host, Opts, "echo.@HOST@"),
     ejabberd_router:register_route(MyHost),
     {ok, #state{host = MyHost}}.
 
@@ -98,7 +117,12 @@ handle_cast(_Msg, State) ->
 %% Description: Handling all non call/cast messages
 %%--------------------------------------------------------------------
 handle_info({route, From, To, Packet}, State) ->
-    ejabberd_router:route(To, From, Packet),
+	Packet2 = case From#jid.user of
+		"" -> jlib:make_error_reply(Packet, ?ERR_BAD_REQUEST);
+		_ -> Packet
+	end,
+    do_client_version(disabled, To, From), % Put 'enabled' to enable it
+    ejabberd_router:route(To, From, Packet2),
     {noreply, State};
 handle_info(_Info, State) ->
     {noreply, State}.
@@ -122,5 +146,58 @@ code_change(_OldVsn, State, _Extra) ->
     {ok, State}.
 
 %%--------------------------------------------------------------------
-%%% Internal functions
+%% Example of routing XMPP packets using Erlang's message passing
 %%--------------------------------------------------------------------
+
+%% To enable this educational example, edit the function handle_info:
+%% replace the argument 'disabled' with 'enabled' in the call to the
+%% function do_client_version.
+
+%% ejabberd provides a method to receive XMPP packets using Erlang's 
+%% message passing mechanism. 
+%%
+%% The packets received by ejabberd are sent
+%% to the local destination process by sending an Erlang message.
+%% This means that you can receive XMPP stanzas in an Erlang process 
+%% using Erlang's Receive, as long as this process is registered in
+%% ejabberd as the process which handles the destination JID.
+%%
+%% This example function is called when a client queries the echo service.
+%% This function then sends a query to the client, and waits 5 seconds to
+%% receive an answer. The answer will only be accepted if it was sent
+%% using exactly the same JID. We add a (mostly) random resource to
+%% try to guarantee that the received response matches the request sent.
+%% Finally, the received response is printed in the ejabberd log file.
+do_client_version(disabled, _From, _To) ->
+    ok;
+do_client_version(enabled, From, To) ->
+    ToS = jlib:jid_to_string(To),
+    %% It is important to identify this process and packet
+    Random_resource = integer_to_list(random:uniform(100000)),
+    From2 = From#jid{resource = Random_resource,
+		     lresource = Random_resource},
+    
+    %% Build an iq:query request
+    Packet = {xmlelement, "iq",
+	      [{"to", ToS}, {"type", "get"}],
+	      [{xmlelement, "query", [{"xmlns", ?NS_VERSION}], []}]},
+    
+    %% Send the request 
+    ejabberd_router:route(From2, To, Packet), 
+    
+    %% Wait to receive the response
+    %% It is very important to only accept a packet which is the 
+    %% response to the request that he sent
+    Els = receive {route, To, From2, IQ} ->
+		  {xmlelement, "query", _, List} = xml:get_subtag(IQ, "query"),
+		  List
+	  after 5000 -> % Timeout in miliseconds: 5 seconds
+		  []
+	  end,
+    Values = [{Name, Value} || {xmlelement,Name,[],[{xmlcdata,Value}]} <- Els],
+    
+    %% Print in log
+    Values_string1 = [io_lib:format("~n~s: ~p", [N, V]) || {N, V} <- Values],
+    Values_string2 = lists:concat(Values_string1),
+    ?INFO_MSG("Information of the client: ~s~s", [ToS, Values_string2]).
+
