@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_irc.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : IRC transport
-%%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_irc.erl 620 2006-09-22 17:01:16Z mremond $
+%%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_irc).
--author('alexey@sevcom.net').
--vsn('$Revision: 620 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_server).
 -behaviour(gen_mod).
@@ -17,8 +34,7 @@
 -export([start_link/2,
 	 start/2,
 	 stop/1,
-	 closed_connection/3,
-	 get_user_and_encoding/3]).
+	 closed_connection/3]).
 
 %% gen_server callbacks
 -export([init/1, handle_call/3, handle_cast/2, handle_info/2,
@@ -27,12 +43,10 @@
 -include("ejabberd.hrl").
 -include("jlib.hrl").
 
--define(DEFAULT_IRC_ENCODING, "koi8-r").
-
 -record(irc_connection, {jid_server_host, pid}).
 -record(irc_custom, {us_host, data}).
 
--record(state, {host, server_host, access}).
+-record(state, {host, server_host, default_encoding, access}).
 
 -define(PROCNAME, ejabberd_mod_irc).
 
@@ -81,15 +95,17 @@ init([Host, Opts]) ->
     mnesia:create_table(irc_custom,
 			[{disc_copies, [node()]},
 			 {attributes, record_info(fields, irc_custom)}]),
-    MyHost = gen_mod:get_opt(host, Opts, "irc." ++ Host),
+    MyHost = gen_mod:get_opt_host(Host, Opts, "irc.@HOST@"),
     update_table(MyHost),
     Access = gen_mod:get_opt(access, Opts, all),
+	DefaultEncoding = gen_mod:get_opt(default_encoding, Opts, "koi8-r"),
     catch ets:new(irc_connection, [named_table,
 				   public,
 				   {keypos, #irc_connection.jid_server_host}]),
     ejabberd_router:register_route(MyHost),
     {ok, #state{host = MyHost,
 		server_host = Host,
+		default_encoding = DefaultEncoding,
 		access = Access}}.
 
 %%--------------------------------------------------------------------
@@ -122,8 +138,9 @@ handle_cast(_Msg, State) ->
 handle_info({route, From, To, Packet},
 	    #state{host = Host,
 		   server_host = ServerHost,
+		   default_encoding = DefEnc,
 		   access = Access} = State) ->
-    case catch do_route(Host, ServerHost, Access, From, To, Packet) of
+    case catch do_route(Host, ServerHost, Access, From, To, Packet, DefEnc) of
 	{'EXIT', Reason} ->
 	    ?ERROR_MSG("~p", [Reason]);
 	_ ->
@@ -171,10 +188,10 @@ stop_supervisor(Host) ->
     supervisor:terminate_child(ejabberd_sup, Proc),
     supervisor:delete_child(ejabberd_sup, Proc).
 
-do_route(Host, ServerHost, Access, From, To, Packet) ->
-    case acl:match_rule(Host, Access, From) of
+do_route(Host, ServerHost, Access, From, To, Packet, DefEnc) ->
+    case acl:match_rule(ServerHost, Access, From) of
 	allow ->
-	    do_route1(Host, ServerHost, From, To, Packet);
+	    do_route1(Host, ServerHost, From, To, Packet, DefEnc);
 	_ ->
 	    {xmlelement, _Name, Attrs, _Els} = Packet,
 	    Lang = xml:get_attr_s("xml:lang", Attrs),
@@ -184,20 +201,20 @@ do_route(Host, ServerHost, Access, From, To, Packet) ->
 	    ejabberd_router:route(To, From, Err)
     end.
 
-do_route1(Host, ServerHost, From, To, Packet) ->
+do_route1(Host, ServerHost, From, To, Packet, DefEnc) ->
     #jid{user = ChanServ, resource = Resource} = To,
-    {xmlelement, _Name, Attrs, _Els} = Packet,
+    {xmlelement, _Name, _Attrs, _Els} = Packet,
     case ChanServ of
 	"" ->
 	    case Resource of
 		"" ->
 		    case jlib:iq_query_info(Packet) of
 			#iq{type = get, xmlns = ?NS_DISCO_INFO = XMLNS,
-			    sub_el = SubEl} = IQ ->
+			    sub_el = _SubEl, lang = Lang} = IQ ->
 			    Res = IQ#iq{type = result,
 					sub_el = [{xmlelement, "query",
 						   [{"xmlns", XMLNS}],
-						   iq_disco()}]},
+						   iq_disco(Lang)}]},
 			    ejabberd_router:route(To,
 						  From,
 						  jlib:iq_to_xml(Res));
@@ -210,7 +227,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 						  From,
 						  jlib:iq_to_xml(Res));
 			#iq{xmlns = ?NS_REGISTER} = IQ ->
-			    process_register(Host, From, To, IQ);
+			    process_register(Host, From, To, DefEnc, IQ);
 			#iq{type = get, xmlns = ?NS_VCARD = XMLNS,
 			    lang = Lang} = IQ ->
 			    Res = IQ#iq{type = result,
@@ -221,7 +238,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
                             ejabberd_router:route(To,
                                                   From,
                                                   jlib:iq_to_xml(Res));
-			#iq{} = IQ ->
+			#iq{} = _IQ ->
 			    Err = jlib:make_error_reply(
 				    Packet, ?ERR_FEATURE_NOT_IMPLEMENTED),
 			    ejabberd_router:route(To, From, Err);
@@ -237,9 +254,9 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 		[[_ | _] = Channel, [_ | _] = Server] ->
 		    case ets:lookup(irc_connection, {From, Server, Host}) of
 			[] ->
-			    io:format("open new connection~n"),
+			    ?DEBUG("open new connection~n", []),
 			    {Username, Encoding} = get_user_and_encoding(
-						     Host, From, Server),
+						     Host, From, Server, DefEnc),
 			    {ok, Pid} = mod_irc_connection:start(
 					  From, Host, ServerHost, Server,
 					  Username, Encoding),
@@ -252,7 +269,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 			    ok;
 			[R] ->
 			    Pid = R#irc_connection.pid,
-			    io:format("send to process ~p~n",
+			    ?DEBUG("send to process ~p~n",
 				      [Pid]),
 			    mod_irc_connection:route_chan(
 			      Pid, Channel, Resource, Packet),
@@ -268,7 +285,7 @@ do_route1(Host, ServerHost, From, To, Packet) ->
 				    ejabberd_router:route(To, From, Err);
 				[R] ->
 				    Pid = R#irc_connection.pid,
-				    io:format("send to process ~p~n",
+				    ?DEBUG("send to process ~p~n",
 					      [Pid]),
 				    mod_irc_connection:route_nick(
 				      Pid, Nick, Packet),
@@ -287,11 +304,11 @@ closed_connection(Host, From, Server) ->
     ets:delete(irc_connection, {From, Server, Host}).
 
 
-iq_disco() ->
+iq_disco(Lang) ->
     [{xmlelement, "identity",
       [{"category", "conference"},
        {"type", "irc"},
-       {"name", "IRC Transport"}], []},
+       {"name", translate:translate(Lang, "IRC Transport")}], []},
      {xmlelement, "feature",
       [{"var", ?NS_MUC}], []},
      {xmlelement, "feature",
@@ -303,14 +320,13 @@ iq_get_vcard(Lang) ->
     [{xmlelement, "FN", [],
       [{xmlcdata, "ejabberd/mod_irc"}]},                  
      {xmlelement, "URL", [],
-      [{xmlcdata,
-        "http://ejabberd.jabberstudio.org/"}]},
+      [{xmlcdata, ?EJABBERD_URI}]},
      {xmlelement, "DESC", [],
-      [{xmlcdata, translate:translate(Lang, "ejabberd IRC module\n"
-        "Copyright (c) 2003-2006 Alexey Shchepin")}]}].
+      [{xmlcdata, translate:translate(Lang, "ejabberd IRC module") ++
+        "\nCopyright (c) 2003-2008 Alexey Shchepin"}]}].
 
-process_register(Host, From, To, #iq{} = IQ) ->
-    case catch process_irc_register(Host, From, To, IQ) of
+process_register(Host, From, To, DefEnc, #iq{} = IQ) ->
+    case catch process_irc_register(Host, From, To, DefEnc, IQ) of
 	{'EXIT', Reason} ->
 	    ?ERROR_MSG("~p", [Reason]);
 	ResIQ ->
@@ -340,7 +356,7 @@ find_xdata_el1([{xmlelement, Name, Attrs, SubEls} | Els]) ->
 find_xdata_el1([_ | Els]) ->
     find_xdata_el1(Els).
 
-process_irc_register(Host, From, To,
+process_irc_register(Host, From, _To, DefEnc,
 		     #iq{type = Type, xmlns = XMLNS,
 			 lang = Lang, sub_el = SubEl} = IQ) ->
     case Type of
@@ -349,7 +365,7 @@ process_irc_register(Host, From, To,
 	    case XDataEl of
 		false ->
 		    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ACCEPTABLE]};
-		{xmlelement, _Name, Attrs, SubEls} ->
+		{xmlelement, _Name, Attrs, _SubEls} ->
 		    case xml:get_attr_s("type", Attrs) of
 			"cancel" ->
 			    IQ#iq{type = result,
@@ -386,7 +402,7 @@ process_irc_register(Host, From, To,
 	get ->
 	    Node =
 		string:tokens(xml:get_tag_attr_s("node", SubEl), "/"),
-	    case get_form(Host, From, Node, Lang) of
+	    case get_form(Host, From, Node, Lang ,DefEnc) of
 		{result, Res} ->
 		    IQ#iq{type = result,
 			  sub_el = [{xmlelement, "query",
@@ -401,13 +417,13 @@ process_irc_register(Host, From, To,
 
 
 
-get_form(Host, From, [], Lang) ->
+get_form(Host, From, [], Lang, DefEnc) ->
     #jid{user = User, server = Server,
 	 luser = LUser, lserver = LServer} = From,
     US = {LUser, LServer},
     Customs =
 	case catch mnesia:dirty_read({irc_custom, {US, Host}}) of
-	    {'EXIT', Reason} ->
+	    {'EXIT', _Reason} ->
 		{error, ?ERR_INTERNAL_SERVER_ERROR};
 	    [] ->
 		{User, []};
@@ -455,7 +471,7 @@ get_form(Host, From, [], Lang) ->
 			   "for IRC servers, fill this list with values "
 			   "in format '{\"irc server\", \"encoding\"}'.  "
 			   "By default this service use \"~s\" encoding."),
-		         [?DEFAULT_IRC_ENCODING]))}]}]},
+		         [DefEnc]))}]}]},
 	        {xmlelement, "field", [{"type", "fixed"}],
 	         [{xmlelement, "value", [],
 		   [{xmlcdata,
@@ -480,13 +496,13 @@ get_form(Host, From, [], Lang) ->
 	       ]}]}
     end;
 
-get_form(_Host, _, _, Lang) ->
+get_form(_Host, _, _, _Lang, _) ->
     {error, ?ERR_SERVICE_UNAVAILABLE}.
 
 
 
 
-set_form(Host, From, [], Lang, XData) ->
+set_form(Host, From, [], _Lang, XData) ->
     {LUser, LServer, _} = jlib:jid_tolower(From),
     US = {LUser, LServer},
     case {lists:keysearch("username", 1, XData),
@@ -526,23 +542,23 @@ set_form(Host, From, [], Lang, XData) ->
     end;
 
 
-set_form(_Host, _, _, Lang, XData) ->
+set_form(_Host, _, _, _Lang, _XData) ->
     {error, ?ERR_SERVICE_UNAVAILABLE}.
 
 
-get_user_and_encoding(Host, From, IRCServer) ->
-    #jid{user = User, server = Server,
+get_user_and_encoding(Host, From, IRCServer, DefEnc) ->
+    #jid{user = User, server = _Server,
 	 luser = LUser, lserver = LServer} = From,
     US = {LUser, LServer},
     case catch mnesia:dirty_read({irc_custom, {US, Host}}) of
-	{'EXIT', Reason} ->
-	    {User, ?DEFAULT_IRC_ENCODING};
+	{'EXIT', _Reason} ->
+	    {User, DefEnc};
 	[] ->
-	    {User, ?DEFAULT_IRC_ENCODING};
+	    {User, DefEnc};
 	[#irc_custom{data = Data}] ->
 	    {xml:get_attr_s(username, Data),
 	     case xml:get_attr_s(IRCServer, xml:get_attr_s(encodings, Data)) of
-		"" -> ?DEFAULT_IRC_ENCODING;
+		"" -> DefEnc;
 		E -> E
 	     end}
     end.
