@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_irc_connection.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : 
-%%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_irc_connection.erl 509 2006-02-18 20:05:05Z alexey $
+%%% Created : 15 Feb 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_irc_connection).
--author('alexey@sevcom.net').
--vsn('$Revision: 509 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_fsm).
 
@@ -180,7 +197,7 @@ code_change(_OldVsn, StateName, StateData, _Extra) ->
 %%          {stop, Reason, NewStateData}                         
 %%----------------------------------------------------------------------
 handle_info({route_chan, Channel, Resource,
-	     {xmlelement, "presence", Attrs, Els}},
+	     {xmlelement, "presence", Attrs, _Els}},
 	    StateName, StateData) ->
     NewStateData =
 	case xml:get_attr_s("type", Attrs) of
@@ -226,7 +243,7 @@ handle_info({route_chan, Channel, Resource,
     end;
 
 handle_info({route_chan, Channel, Resource,
-	     {xmlelement, "message", Attrs, Els} = El},
+	     {xmlelement, "message", Attrs, _Els} = El},
 	    StateName, StateData) ->
     NewStateData =
 	case xml:get_attr_s("type", Attrs) of
@@ -348,7 +365,7 @@ handle_info({route_chan, Channel, Resource,
 
 
 handle_info({route_chan, Channel, Resource,
-	     {xmlelement, "iq", Attrs, Els} = El},
+	     {xmlelement, "iq", _Attrs, _Els} = El},
 	    StateName, StateData) ->
     From = StateData#state.user,
     To = jlib:make_jid(lists:concat([Channel, "%", StateData#state.server]),
@@ -386,12 +403,12 @@ handle_info({route_chan, Channel, Resource,
     end,
     {next_state, StateName, StateData};
 
-handle_info({route_chan, Channel, Resource, Packet}, StateName, StateData) ->
+handle_info({route_chan, _Channel, _Resource, _Packet}, StateName, StateData) ->
     {next_state, StateName, StateData};
 
 
 handle_info({route_nick, Nick,
-	     {xmlelement, "message", Attrs, Els} = El},
+	     {xmlelement, "message", Attrs, _Els} = El},
 	    StateName, StateData) ->
     NewStateData =
 	case xml:get_attr_s("type", Attrs) of
@@ -446,7 +463,7 @@ handle_info({route_nick, Nick,
 	    {next_state, StateName, NewStateData}
     end;
 
-handle_info({route_nick, Nick, Packet}, StateName, StateData) ->
+handle_info({route_nick, _Nick, _Packet}, StateName, StateData) ->
     {next_state, StateName, StateData};
 
 
@@ -454,16 +471,16 @@ handle_info({ircstring, [$P, $I, $N, $G, $  | ID]}, StateName, StateData) ->
     send_text(StateData, "PONG " ++ ID ++ "\r\n"),
     {next_state, StateName, StateData};
 
-handle_info({ircstring, [$: | String]}, StateName, StateData) ->
+handle_info({ircstring, [$: | String]}, _StateName, StateData) ->
     Words = string:tokens(String, " "),
     NewStateData =
 	case Words of
 	    [_, "353" | Items] ->
 		process_channel_list(StateData, Items);
-	    [_, "332", Nick, [$# | Chan] | _] ->
+	    [_, "332", _Nick, [$# | Chan] | _] ->
 		process_channel_topic(StateData, Chan, String),
 		StateData;
-	    [_, "333", Nick, [$# | Chan] | _] ->
+	    [_, "333", _Nick, [$# | Chan] | _] ->
 		process_channel_topic_who(StateData, Chan, String),
 		StateData;
 	    [_, "318", _, Nick | _] ->
@@ -546,16 +563,16 @@ handle_info({ircstring, String}, StateName, StateData) ->
 handle_info({send_text, Text}, StateName, StateData) ->
     send_text(StateData, Text),
     {next_state, StateName, StateData};
-handle_info({tcp, Socket, Data}, StateName, StateData) ->
+handle_info({tcp, _Socket, Data}, StateName, StateData) ->
     Buf = StateData#state.inbuf ++ binary_to_list(Data),
     {ok, Strings} = regexp:split([C || C <- Buf, C /= $\r], "\n"),
     ?DEBUG("strings=~p~n", [Strings]),
     NewBuf = process_lines(StateData#state.encoding, Strings),
     {next_state, StateName, StateData#state{inbuf = NewBuf}};
-handle_info({tcp_closed, Socket}, StateName, StateData) ->
+handle_info({tcp_closed, _Socket}, StateName, StateData) ->
     gen_fsm:send_event(self(), closed),
     {next_state, StateName, StateData};
-handle_info({tcp_error, Socket, Reason}, StateName, StateData) ->
+handle_info({tcp_error, _Socket, _Reason}, StateName, StateData) ->
     gen_fsm:send_event(self(), closed),
     {next_state, StateName, StateData}.
 
@@ -564,7 +581,7 @@ handle_info({tcp_error, Socket, Reason}, StateName, StateData) ->
 %% Purpose: Shutdown the fsm
 %% Returns: any
 %%----------------------------------------------------------------------
-terminate(Reason, StateName, StateData) ->
+terminate(_Reason, _StateName, StateData) ->
     mod_irc:closed_connection(StateData#state.host,
 			      StateData#state.user,
 			      StateData#state.server),
@@ -610,7 +627,7 @@ send_text(#state{socket = Socket, encoding = Encoding}, Text) ->
 bounce_messages(Reason) ->
     receive
 	{send_element, El} ->
-	    {xmlelement, Name, Attrs, SubTags} = El,
+	    {xmlelement, _Name, Attrs, _SubTags} = El,
 	    case xml:get_attr_s("type", Attrs) of
 	        "error" ->
 	            ok;
@@ -634,7 +651,7 @@ route_nick(Pid, Nick, Packet) ->
     Pid ! {route_nick, Nick, Packet}.
 
 
-process_lines(Encoding, [S]) ->
+process_lines(_Encoding, [S]) ->
     S;
 process_lines(Encoding, [S | Ss]) ->
     self() ! {ircstring, iconv:convert(Encoding, "utf-8", S)},
@@ -650,7 +667,7 @@ process_channel_list_find_chan(StateData, [[$# | Chan] | Items]) ->
 process_channel_list_find_chan(StateData, [_ | Items]) ->
     process_channel_list_find_chan(StateData, Items).
 
-process_channel_list_users(StateData, Chan, []) ->
+process_channel_list_users(StateData, _Chan, []) ->
     StateData;
 process_channel_list_users(StateData, Chan, [User | Items]) ->
     NewStateData = process_channel_list_user(StateData, Chan, User),
@@ -709,7 +726,7 @@ process_channel_topic_who(StateData, Chan, String) ->
     Msg1 = case Words of
 	       [_, "333", _, _Chan, Whoset , Timeset] ->
 		   case string:to_integer(Timeset) of
-		       {Unixtimeset, Rest} -> 
+		       {Unixtimeset, _Rest} -> 
 			   "Topic for #" ++ Chan ++ " set by " ++ Whoset ++
 			       " at " ++ unixtime2string(Unixtimeset);
 		       _->
@@ -731,7 +748,7 @@ process_channel_topic_who(StateData, Chan, String) ->
 
 
 
-process_endofwhois(StateData, String, Nick) ->
+process_endofwhois(StateData, _String, Nick) ->
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Nick, "!", StateData#state.server]),
 		    StateData#state.host, ""),
@@ -814,7 +831,7 @@ process_channotice(StateData, Chan, From, String) ->
 
 
 
-process_privmsg(StateData, Nick, From, String) ->
+process_privmsg(StateData, _Nick, From, String) ->
     [FromUser | _] = string:tokens(From, "!"),
     {ok, Msg, _} = regexp:sub(String, ".*PRIVMSG[^:]*:", ""),
     Msg1 = case Msg of
@@ -832,7 +849,7 @@ process_privmsg(StateData, Nick, From, String) ->
        [{xmlelement, "body", [], [{xmlcdata, Msg2}]}]}).
 
 
-process_notice(StateData, Nick, From, String) ->
+process_notice(StateData, _Nick, From, String) ->
     [FromUser | _] = string:tokens(From, "!"),
     {ok, Msg, _} = regexp:sub(String, ".*NOTICE[^:]*:", ""),
     Msg1 = case Msg of
@@ -850,7 +867,7 @@ process_notice(StateData, Nick, From, String) ->
        [{xmlelement, "body", [], [{xmlcdata, Msg2}]}]}).
 
 
-process_version(StateData, Nick, From) ->
+process_version(StateData, _Nick, From) ->
     [FromUser | _] = string:tokens(From, "!"),
     send_text(
       StateData,
@@ -859,12 +876,12 @@ process_version(StateData, Nick, From) ->
 		    "\001\r\n",
 		    [FromUser, ?VERSION]) ++
       io_lib:format("NOTICE ~s :\001VERSION "
-		    "http://ejabberd.jabberstudio.org/"
+		    ?EJABBERD_URI
 		    "\001\r\n",
 		    [FromUser])).
 
 
-process_userinfo(StateData, Nick, From) ->
+process_userinfo(StateData, _Nick, From) ->
     [FromUser | _] = string:tokens(From, "!"),
     send_text(
       StateData,
@@ -922,7 +939,7 @@ process_quit(StateData, From, String) ->
     
     {ok, Msg, _} = regexp:sub(String, ".*QUIT[^:]*:", ""),
     Msg1 = filter_message(Msg),
-    NewChans =
+    %%NewChans =
 	dict:map(
 	  fun(Chan, Ps) ->
 		  case ?SETS:is_member(FromUser, Ps) of
@@ -949,7 +966,7 @@ process_quit(StateData, From, String) ->
     StateData.
 
 
-process_join(StateData, Channel, From, String) ->
+process_join(StateData, Channel, From, _String) ->
     [FromUser | FromIdent] = string:tokens(From, "!"),
     Chan = lists:subtract(Channel, ":#"),
     ejabberd_router:route(
@@ -977,7 +994,7 @@ process_join(StateData, Channel, From, String) ->
 
 
 
-process_mode_o(StateData, Chan, From, Nick, Affiliation, Role) ->
+process_mode_o(StateData, Chan, _From, Nick, Affiliation, Role) ->
     %Msg = lists:last(string:tokens(String, ":")),
     ejabberd_router:route(
       jlib:make_jid(lists:concat([Chan, "%", StateData#state.server]),
@@ -1119,15 +1136,15 @@ process_iq_admin(StateData, Channel, set, SubEl) ->
 	    Reason = xml:get_path_s(ItemEl, [{elem, "reason"}, cdata]),
 	    process_admin(StateData, Channel, Nick, Affiliation, Role, Reason)
     end;
-process_iq_admin(StateData, Channel, get, SubEl) ->
+process_iq_admin(_StateData, _Channel, get, _SubEl) ->
     {error, ?ERR_FEATURE_NOT_IMPLEMENTED}.
 
 
 
-process_admin(StateData, Channel, "", Affiliation, Role, Reason) ->
+process_admin(_StateData, _Channel, "", _Affiliation, _Role, _Reason) ->
     {error, ?ERR_FEATURE_NOT_IMPLEMENTED};
 
-process_admin(StateData, Channel, Nick, Affiliation, "none", Reason) ->
+process_admin(StateData, Channel, Nick, _Affiliation, "none", Reason) ->
     case Reason of
 	"" ->
 	    send_text(StateData,
@@ -1142,7 +1159,7 @@ process_admin(StateData, Channel, Nick, Affiliation, "none", Reason) ->
 
 
 
-process_admin(StateData, Channel, Nick, Affiliation, Role, Reason) ->
+process_admin(_StateData, _Channel, _Nick, _Affiliation, _Role, _Reason) ->
     {error, ?ERR_FEATURE_NOT_IMPLEMENTED}.
 
 
