@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_last.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : jabber:iq:last support (JEP-0012)
-%%% Created : 24 Oct 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_last.erl 370 2005-06-20 03:18:13Z alexey $
+%%% Created : 24 Oct 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_last).
--author('alexey@sevcom.net').
--vsn('$Revision: 370 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_mod).
 
@@ -18,10 +35,12 @@
 	 process_sm_iq/3,
 	 on_presence_update/4,
 	 store_last_info/4,
+	 get_last_info/2,
 	 remove_user/2]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
+-include("mod_privacy.hrl").
 
 -record(last_activity, {us, timestamp, status}).
 
@@ -76,23 +95,22 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 		  {none, []}, [User, Server, From]),
 	    if
 		(Subscription == both) or (Subscription == from) ->
-		    case catch mod_privacy:get_user_list(User, Server) of
-			{'EXIT', _Reason} ->
+		    UserListRecord = ejabberd_hooks:run_fold(
+				       privacy_get_user_list, Server,
+				       #userlist{},
+				       [User, Server]),
+		    case ejabberd_hooks:run_fold(
+			   privacy_check_packet, Server,
+			   allow,
+			   [User, Server, UserListRecord,
+			    {From, To,
+			     {xmlelement, "presence", [], []}},
+			    out]) of
+			allow ->
 			    get_last(IQ, SubEl, User, Server);
-			List ->
-			    case catch mod_privacy:check_packet(
-					 User, Server, List,
-					 {From, To,
-					  {xmlelement, "presence", [], []}},
-					 out) of
-				{'EXIT', _Reason} ->
-				    get_last(IQ, SubEl, User, Server);
-				allow ->
-				    get_last(IQ, SubEl, User, Server);
-				deny ->
-				    IQ#iq{type = error,
-					  sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
-			    end
+			deny ->
+			    IQ#iq{type = error,
+				  sub_el = [SubEl, ?ERR_NOT_ALLOWED]}
 		    end;
 		true ->
 		    IQ#iq{type = error,
@@ -100,6 +118,7 @@ process_sm_iq(From, To, #iq{type = Type, sub_el = SubEl} = IQ) ->
 	    end
     end.
 
+%% TODO: This function could use get_last_info/2
 get_last(IQ, SubEl, LUser, LServer) ->
     case catch mnesia:dirty_read(last_activity, {LUser, LServer}) of
 	{'EXIT', _Reason} ->
@@ -135,6 +154,16 @@ store_last_info(User, Server, TimeStamp, Status) ->
 	end,
     mnesia:transaction(F).
     
+%% Returns: {ok, Timestamp, Status} | not_found
+get_last_info(LUser, LServer) ->
+    case catch mnesia:dirty_read(last_activity, {LUser, LServer}) of
+	{'EXIT', _Reason} ->
+	    not_found;
+	[] ->
+	    not_found;
+	[#last_activity{timestamp = TimeStamp, status = Status}] ->
+	    {ok, TimeStamp, Status}
+    end.
 
 remove_user(User, Server) ->
     LUser = jlib:nodeprep(User),
