@@ -1,35 +1,57 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_offline.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : 
-%%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_offline.erl 569 2006-05-23 20:19:37Z mremond $
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
+%%% Purpose : Store and manage offline messages in Mnesia database.
+%%% Created :  5 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_offline).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 -behaviour(gen_mod).
 
 -export([start/2,
-	 init/0,
+	 init/1,
 	 stop/1,
 	 store_packet/3,
 	 resend_offline_messages/2,
 	 pop_offline_messages/3,
 	 remove_expired_messages/0,
 	 remove_old_messages/1,
-	 remove_user/2]).
+	 remove_user/2,
+	 webadmin_page/3,
+	 webadmin_user/4]).
 
 -include("ejabberd.hrl").
 -include("jlib.hrl").
+-include("web/ejabberd_http.hrl").
+-include("web/ejabberd_web_admin.hrl").
 
 -record(offline_msg, {us, timestamp, expire, from, to, packet}).
 
 -define(PROCNAME, ejabberd_offline).
 -define(OFFLINE_TABLE_LOCK_THRESHOLD, 1000).
 
-start(Host, _Opts) ->
+start(Host, Opts) ->
     mnesia:create_table(offline_msg,
 			[{disc_only_copies, [node()]},
 			 {type, bag},
@@ -41,38 +63,62 @@ start(Host, _Opts) ->
 		       ?MODULE, pop_offline_messages, 50),
     ejabberd_hooks:add(remove_user, Host,
 		       ?MODULE, remove_user, 50),
+    ejabberd_hooks:add(anonymous_purge_hook, Host,
+		       ?MODULE, remove_user, 50),
+    ejabberd_hooks:add(webadmin_page_host, Host,
+		       ?MODULE, webadmin_page, 50),
+    ejabberd_hooks:add(webadmin_user, Host,
+		       ?MODULE, webadmin_user, 50),
+    MaxOfflineMsgs = gen_mod:get_opt(user_max_messages, Opts, infinity),
     register(gen_mod:get_module_proc(Host, ?PROCNAME),
-	     spawn(?MODULE, init, [])).
+	     spawn(?MODULE, init, [MaxOfflineMsgs])).
 
-init() ->
-    loop().
+%% MaxOfflineMsgs is either infinity of integer > 0
+init(infinity) ->
+    loop(infinity);
+init(MaxOfflineMsgs) 
+  when integer(MaxOfflineMsgs), MaxOfflineMsgs > 0 ->
+    loop(MaxOfflineMsgs).
 
-loop() ->
+loop(MaxOfflineMsgs) ->
     receive
-	#offline_msg{} = Msg ->
-	    Msgs = receive_all([Msg]),
+	#offline_msg{us=US} = Msg ->
+	    Msgs = receive_all(US, [Msg]),
 	    Len = length(Msgs),
 	    F = fun() ->
+			%% Only count messages if needed:
+			Count = if MaxOfflineMsgs =/= infinity ->
+					Len + p1_mnesia:count_records(
+						offline_msg, 
+						#offline_msg{us=US, _='_'});
+				   true -> 
+					0
+				end,
 			if
-			    Len >= ?OFFLINE_TABLE_LOCK_THRESHOLD ->
-				mnesia:write_lock_table(offline_msg);
+			    Count > MaxOfflineMsgs ->
+				discard_warn_sender(Msgs);
 			    true ->
-				ok
-			end,
-			lists:foreach(fun(M) ->
-					      mnesia:write(M)
-				      end, Msgs)
+				if
+				    Len >= ?OFFLINE_TABLE_LOCK_THRESHOLD ->
+					mnesia:write_lock_table(offline_msg);
+				    true ->
+					ok
+				end,
+				lists:foreach(fun(M) ->
+						      mnesia:write(M)
+					      end, Msgs)
+			end
 		end,
 	    mnesia:transaction(F),
-	    loop();
+	    loop(MaxOfflineMsgs);
 	_ ->
-	    loop()
+	    loop(MaxOfflineMsgs)
     end.
 
-receive_all(Msgs) ->
+receive_all(US, Msgs) ->
     receive
-	#offline_msg{} = Msg ->
-	    receive_all([Msg | Msgs])
+	#offline_msg{us=US} = Msg ->
+	    receive_all(US, [Msg | Msgs])
     after 0 ->
 	    Msgs
     end.
@@ -85,6 +131,12 @@ stop(Host) ->
 			  ?MODULE, pop_offline_messages, 50),
     ejabberd_hooks:delete(remove_user, Host,
 			  ?MODULE, remove_user, 50),
+    ejabberd_hooks:delete(anonymous_purge_hook, Host,
+			  ?MODULE, remove_user, 50),
+    ejabberd_hooks:delete(webadmin_page_host, Host,
+			  ?MODULE, webadmin_page, 50),
+    ejabberd_hooks:delete(webadmin_user, Host,
+			  ?MODULE, webadmin_user, 50),
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     exit(whereis(Proc), stop),
     {wait, Proc}.
@@ -167,20 +219,16 @@ find_x_expire(TimeStamp, [{xmlcdata, _} | Els]) ->
 find_x_expire(TimeStamp, [El | Els]) ->
     case xml:get_tag_attr_s("xmlns", El) of
 	?NS_EXPIRE ->
-	    case xml:get_tag_attr_s("seconds", El) of
-		Val ->
-		    case catch list_to_integer(Val) of
-		        {'EXIT', _} ->
-			    never;
-			Int when Int > 0 ->
-			    {MegaSecs, Secs, MicroSecs} = TimeStamp,
-			    S = MegaSecs * 1000000 + Secs + Int,
-			    MegaSecs1 = S div 1000000,
-			    Secs1 = S rem 1000000,
-			    {MegaSecs1, Secs1, MicroSecs};
-			_ ->
-			    never
-		    end;
+	    Val = xml:get_tag_attr_s("seconds", El),
+	    case catch list_to_integer(Val) of
+		{'EXIT', _} ->
+		    never;
+		Int when Int > 0 ->
+		    {MegaSecs, Secs, MicroSecs} = TimeStamp,
+		    S = MegaSecs * 1000000 + Secs + Int,
+		    MegaSecs1 = S div 1000000,
+		    Secs1 = S rem 1000000,
+		    {MegaSecs1, Secs1, MicroSecs};
 		_ ->
 		    never
 	    end;
@@ -388,3 +436,122 @@ update_table() ->
 	    mnesia:transform_table(offline_msg, ignore, Fields)
     end.
 
+
+%% Helper functions:
+
+%% Warn senders that their messages have been discarded:
+discard_warn_sender(Msgs) ->
+    lists:foreach(
+      fun(#offline_msg{from=From, to=To, packet=Packet}) ->
+	      ErrText = "Your contact offline message queue is full. The message has been discarded.",
+	      Lang = xml:get_tag_attr_s("xml:lang", Packet),
+	      Err = jlib:make_error_reply(
+		      Packet, ?ERRT_RESOURCE_CONSTRAINT(Lang, ErrText)),
+	      ejabberd_router:route(
+		To,
+		From, Err)
+      end, Msgs).
+
+
+webadmin_page(_, Host,
+	      #request{us = _US,
+		       path = ["user", U, "queue"],
+		       q = Query,
+		       lang = Lang} = _Request) ->
+    Res = user_queue(U, Host, Query, Lang),
+    {stop, Res};
+
+webadmin_page(Acc, _, _) -> Acc.
+
+user_queue(User, Server, Query, Lang) ->
+    US = {jlib:nodeprep(User), jlib:nameprep(Server)},
+    Res = user_queue_parse_query(US, Query),
+    Msgs = lists:keysort(#offline_msg.timestamp,
+			 mnesia:dirty_read({offline_msg, US})),
+    FMsgs =
+	lists:map(
+	  fun(#offline_msg{timestamp = TimeStamp, from = From, to = To,
+			   packet = {xmlelement, Name, Attrs, Els}} = Msg) ->
+		  ID = jlib:encode_base64(binary_to_list(term_to_binary(Msg))),
+		  {{Year, Month, Day}, {Hour, Minute, Second}} =
+		      calendar:now_to_local_time(TimeStamp),
+		  Time = lists:flatten(
+			   io_lib:format(
+			     "~w-~.2.0w-~.2.0w ~.2.0w:~.2.0w:~.2.0w",
+			     [Year, Month, Day, Hour, Minute, Second])),
+		  SFrom = jlib:jid_to_string(From),
+		  STo = jlib:jid_to_string(To),
+		  Attrs2 = jlib:replace_from_to_attrs(SFrom, STo, Attrs),
+		  Packet = {xmlelement, Name, Attrs2, Els},
+		  FPacket = ejabberd_web_admin:pretty_print_xml(Packet),
+		  ?XE("tr",
+		      [?XAE("td", [{"class", "valign"}], [?INPUT("checkbox", "selected", ID)]),
+		       ?XAC("td", [{"class", "valign"}], Time),
+		       ?XAC("td", [{"class", "valign"}], SFrom),
+		       ?XAC("td", [{"class", "valign"}], STo),
+		       ?XAE("td", [{"class", "valign"}], [?XC("pre", FPacket)])]
+		     )
+	  end, Msgs),
+    [?XC("h1", io_lib:format(?T("~s's Offline Messages Queue"),
+			     [us_to_list(US)]))] ++
+	case Res of
+	    ok -> [?CT("Submitted"), ?P];
+	    nothing -> []
+	end ++
+	[?XAE("form", [{"action", ""}, {"method", "post"}],
+	      [?XE("table",
+		   [?XE("thead",
+			[?XE("tr",
+			     [?X("td"),
+			      ?XCT("td", "Time"),
+			      ?XCT("td", "From"),
+			      ?XCT("td", "To"),
+			      ?XCT("td", "Packet")
+			     ])]),
+		    ?XE("tbody",
+			if
+			    FMsgs == [] ->
+				[?XE("tr",
+				     [?XAC("td", [{"colspan", "4"}], " ")]
+				    )];
+			    true ->
+				FMsgs
+			end
+		       )]),
+	       ?BR,
+	       ?INPUTT("submit", "delete", "Delete Selected")
+	      ])].
+
+user_queue_parse_query(US, Query) ->
+    case lists:keysearch("delete", 1, Query) of
+	{value, _} ->
+	    Msgs = lists:keysort(#offline_msg.timestamp,
+				 mnesia:dirty_read({offline_msg, US})),
+	    F = fun() ->
+			lists:foreach(
+			  fun(Msg) ->
+				  ID = jlib:encode_base64(
+					 binary_to_list(term_to_binary(Msg))),
+				  case lists:member({"selected", ID}, Query) of
+				      true ->
+					  mnesia:delete_object(Msg);
+				      false ->
+					  ok
+				  end
+			  end, Msgs)
+		end,
+	    mnesia:transaction(F),
+	    ok;
+	false ->
+	    nothing
+    end.
+
+us_to_list({User, Server}) ->
+    jlib:jid_to_string({User, Server, ""}).
+
+webadmin_user(Acc, User, Server, Lang) ->
+    US = {jlib:nodeprep(User), jlib:nameprep(Server)},
+    QueueLen = length(mnesia:dirty_read({offline_msg, US})),
+    FQueueLen = [?AC("queue/",
+		     integer_to_list(QueueLen))],
+    Acc ++ [?XCT("h3", "Offline Messages:")] ++ FQueueLen.
