@@ -1,20 +1,38 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_register.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Inband registration support
-%%% Created :  8 Dec 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_register.erl 590 2006-07-28 16:18:50Z mremond $
+%%% Created :  8 Dec 2002 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_register).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 -behaviour(gen_mod).
 
 -export([start/2,
 	 stop/1,
 	 stream_feature_register/1,
-	 unauthenticated_iq_register/3,
+	 unauthenticated_iq_register/4,
 	 process_iq/3]).
 
 -include("ejabberd.hrl").
@@ -30,6 +48,11 @@ start(Host, Opts) ->
  		       ?MODULE, stream_feature_register, 50),
     ejabberd_hooks:add(c2s_unauthenticated_iq, Host,
  		       ?MODULE, unauthenticated_iq_register, 50),
+    mnesia:create_table(mod_register_ip,
+			[{ram_copies, [node()]},
+			 {local_content, true},
+			 {attributes, [key, value]}]),
+    mnesia:add_table_copy(mod_register_ip, node(), ram_copies),
     ok.
 
 stop(Host) ->
@@ -45,20 +68,30 @@ stream_feature_register(Acc) ->
     [{xmlelement, "register",
       [{"xmlns", ?NS_FEATURE_IQREGISTER}], []} | Acc].
 
-unauthenticated_iq_register(_Acc, Server, #iq{xmlns = ?NS_REGISTER} = IQ) ->
+unauthenticated_iq_register(_Acc,
+			    Server, #iq{xmlns = ?NS_REGISTER} = IQ, IP) ->
+    Address = case IP of
+		 {A, _Port} -> A;
+		  _ -> undefined
+	      end,
     ResIQ = process_iq(jlib:make_jid("", "", ""),
  		       jlib:make_jid("", Server, ""),
- 		       IQ),
+ 		       IQ,
+		       Address),
     Res1 = jlib:replace_from_to(jlib:make_jid("", Server, ""),
  				jlib:make_jid("", "", ""),
  				jlib:iq_to_xml(ResIQ)),
     jlib:remove_attr("to", Res1);
 
-unauthenticated_iq_register(Acc, _Server, _IQ) ->
+unauthenticated_iq_register(Acc, _Server, _IQ, _IP) ->
     Acc.
 
+process_iq(From, To, IQ) ->
+    process_iq(From, To, IQ, jlib:jid_tolower(jlib:jid_remove_resource(From))).
+
 process_iq(From, To,
-	   #iq{type = Type, lang = Lang, sub_el = SubEl, id = ID} = IQ) ->
+	   #iq{type = Type, lang = Lang, sub_el = SubEl, id = ID} = IQ,
+	   Source) ->
     case Type of
 	set ->
 	    UTag = xml:get_subtag(SubEl, "username"),
@@ -82,6 +115,12 @@ process_iq(From, To,
 					ok ->
 					    IQ#iq{type = result,
 						  sub_el = [SubEl]};
+					%% TODO FIXME: This piece of
+					%% code does not work since
+					%% the code have been changed
+					%% to allow several auth
+					%% modules.  lists:foreach can
+					%% only return ok:
 					not_allowed ->
 					    IQ#iq{type = error,
 						  sub_el =
@@ -127,7 +166,8 @@ process_iq(From, To,
 			    ejabberd_auth:set_password(User, Server, Password),
 			    IQ#iq{type = result, sub_el = [SubEl]};
 			_ ->
-			    case try_register(User, Server, Password) of
+			    case try_register(User, Server, Password,
+					      Source) of
 				ok ->
 				    IQ#iq{type = result, sub_el = [SubEl]};
 				{error, Error} ->
@@ -155,7 +195,7 @@ process_iq(From, To,
     end.
 
 
-try_register(User, Server, Password) ->
+try_register(User, Server, Password, Source) ->
     case jlib:is_nodename(User) of
 	false ->
 	    {error, ?ERR_BAD_REQUEST};
@@ -166,19 +206,30 @@ try_register(User, Server, Password) ->
 		deny ->
 		    {error, ?ERR_CONFLICT};
 		allow ->
-		    case ejabberd_auth:try_register(User, Server, Password) of
-			{atomic, ok} ->
-			    send_welcome_message(JID),
-			    send_registration_notifications(JID),
-			    ok;
-			{atomic, exists} ->
-			    {error, ?ERR_CONFLICT};
-			{error, invalid_jid} ->
-			    {error, ?ERR_JID_MALFORMED};
-			{error, not_allowed} ->
-			    {error, ?ERR_NOT_ALLOWED};
-			{error, _Reason} ->
-			    {error, ?ERR_INTERNAL_SERVER_ERROR}
+		    case check_timeout(Source) of
+			true ->
+			    case ejabberd_auth:try_register(User, Server, Password) of
+				{atomic, ok} ->
+				    ejabberd_hooks:run(user_registered, Server,
+						       [User, Server]),
+				    send_welcome_message(JID),
+				    send_registration_notifications(JID),
+				    ok;
+				Error ->
+				    remove_timeout(Source),
+				    case Error of
+					{atomic, exists} ->
+					    {error, ?ERR_CONFLICT};
+					{error, invalid_jid} ->
+					    {error, ?ERR_JID_MALFORMED};
+					{error, not_allowed} ->
+					    {error, ?ERR_NOT_ALLOWED};
+					{error, _Reason} ->
+					    {error, ?ERR_INTERNAL_SERVER_ERROR}
+				    end
+			    end;
+			false ->
+			    {error, ?ERR_RESOURCE_CONSTRAINT}
 		    end
 	    end
     end.
@@ -186,7 +237,7 @@ try_register(User, Server, Password) ->
 
 send_welcome_message(JID) ->
     Host = JID#jid.lserver,
-    case ejabberd_config:get_local_option({welcome_message, Host}) of
+    case gen_mod:get_module_opt(Host, ?MODULE, welcome_message, {"", ""}) of
 	{"", ""} ->
 	    ok;
 	{Subj, Body} ->
@@ -202,7 +253,7 @@ send_welcome_message(JID) ->
 
 send_registration_notifications(UJID) ->
     Host = UJID#jid.lserver,
-    case ejabberd_config:get_local_option({registration_watchers, Host}) of
+    case gen_mod:get_module_opt(Host, ?MODULE, registration_watchers, []) of
 	[] -> ok;
 	JIDs when is_list(JIDs) ->
 	    Body = lists:flatten(
@@ -226,3 +277,93 @@ send_registration_notifications(UJID) ->
 	    ok
     end.
 
+
+check_timeout(undefined) ->
+    true;
+check_timeout(Source) ->
+    Timeout = case ejabberd_config:get_local_option(registration_timeout) of
+		  undefined -> 600;
+		  TO -> TO
+	      end,
+    if
+	is_integer(Timeout) ->
+	    {MSec, Sec, _USec} = now(),
+	    Priority = -(MSec * 1000000 + Sec),
+	    CleanPriority = Priority + Timeout,
+	    F = fun() ->
+			Treap = case mnesia:read(mod_register_ip, treap,
+						 write) of
+				    [] ->
+					treap:empty();
+				    [{mod_register_ip, treap, T}] -> T
+				end,
+			Treap1 = clean_treap(Treap, CleanPriority),
+			case treap:lookup(Source, Treap1) of
+			    error ->
+				Treap2 = treap:insert(Source, Priority, [],
+						      Treap1),
+				mnesia:write({mod_register_ip, treap, Treap2}),
+				true;
+			    {ok, _, _} ->
+				mnesia:write({mod_register_ip, treap, Treap1}),
+				false
+			end
+		end,
+	    case mnesia:transaction(F) of
+		{atomic, Res} ->
+		    Res;
+		{aborted, Reason} ->
+		    ?ERROR_MSG("mod_register: timeout check error: ~p~n",
+			       [Reason]),
+		    true
+	    end;
+	true ->
+	    true
+    end.
+
+clean_treap(Treap, CleanPriority) ->
+    case treap:is_empty(Treap) of
+	true ->
+	    Treap;
+	false ->
+	    {_Key, Priority, _Value} = treap:get_root(Treap),
+	    if
+		Priority > CleanPriority ->
+		    clean_treap(treap:delete_root(Treap), CleanPriority);
+		true ->
+		    Treap
+	    end
+    end.
+
+remove_timeout(undefined) ->
+    true;
+remove_timeout(Source) ->
+    Timeout = case ejabberd_config:get_local_option(registration_timeout) of
+		  undefined -> 600;
+		  TO -> TO
+	      end,
+    if
+	is_integer(Timeout) ->
+	    F = fun() ->
+			Treap = case mnesia:read(mod_register_ip, treap,
+						 write) of
+				    [] ->
+					treap:empty();
+				    [{mod_register_ip, treap, T}] -> T
+				end,
+			Treap1 = treap:delete(Source, Treap),
+			mnesia:write({mod_register_ip, treap, Treap1}),
+			ok
+		end,
+	    case mnesia:transaction(F) of
+		{atomic, ok} ->
+		    ok;
+		{aborted, Reason} ->
+		    ?ERROR_MSG("mod_register: timeout remove error: ~p~n",
+			       [Reason]),
+		    ok
+	    end;
+	true ->
+	    ok
+    end.
+
