@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : mod_vcard_ldap.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Support for VCards from LDAP storage.
-%%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: mod_vcard_ldap.erl 623 2006-09-23 09:52:53Z mremond $
+%%% Created :  2 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(mod_vcard_ldap).
--author('alexey@sevcom.net').
--vsn('$Revision: 623 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_server).
 -behaviour(gen_mod).
@@ -28,7 +45,8 @@
 	 get_sm_features/5,
 	 process_local_iq/3,
 	 process_sm_iq/3,
-	 remove_user/1
+	 remove_user/1,
+	 route/4
 	]).
 
 -include("ejabberd.hrl").
@@ -42,19 +60,20 @@
 		eldap_id,
 		search,
 		servers,
+		backups,
 		port,
 		dn,
 		base,
 		password,
-		uid,
-		uid_format,
+		uids,
 		vcard_map,
 		vcard_map_attrs,
 		user_filter,
 		search_filter,
 		search_fields,
 		search_reported,
-		search_reported_attrs
+		search_reported_attrs,
+		matches
 	       }).
 
 -define(VCARD_MAP,
@@ -97,10 +116,10 @@
 
 -define(SEARCH_REPORTED,
 	[{"Full Name", "FN"},
-	 {"Given Name", "GIVEN"},
+	 {"Given Name", "FIRST"},
 	 {"Middle Name", "MIDDLE"},
-	 {"Family Name", "FAMILY"},
-	 {"Nickname", "NICKNAME"},
+	 {"Family Name", "LAST"},
+	 {"Nickname", "NICK"},
 	 {"Birthday", "BDAY"},
 	 {"Country", "CTRY"},
 	 {"City", "LOCALITY"},
@@ -121,7 +140,7 @@ start(Host, Opts) ->
     Proc = gen_mod:get_module_proc(Host, ?PROCNAME),
     ChildSpec = {
       Proc, {?MODULE, start_link, [Host, Opts]},
-      permanent, 1000, worker, [?MODULE]
+      transient, 1000, worker, [?MODULE]
      },
     supervisor:start_child(ejabberd_sup, ChildSpec).
 
@@ -149,14 +168,15 @@ start_link(Host, Opts) ->
 
 init([Host, Opts]) ->
     State = parse_options(Host, Opts),
-    IQDisc = gen_mod:get_opt(iqdisc, Opts, one_queue),
+    IQDisc = gen_mod:get_opt(iqdisc, Opts, parallel),
     gen_iq_handler:add_iq_handler(ejabberd_local, Host, ?NS_VCARD,
 				  ?MODULE, process_local_iq, IQDisc),
     gen_iq_handler:add_iq_handler(ejabberd_sm, Host, ?NS_VCARD,
 				  ?MODULE, process_sm_iq, IQDisc),
     ejabberd_hooks:add(disco_sm_features, Host, ?MODULE, get_sm_features, 50),
-    eldap:start_link(State#state.eldap_id,
+    eldap_pool:start_link(State#state.eldap_id,
 		     State#state.servers,
+		     State#state.backups,
 		     State#state.port,
 		     State#state.dn,
 		     State#state.password),
@@ -170,14 +190,13 @@ init([Host, Opts]) ->
 
 handle_info({route, From, To, Packet}, State) ->
     case catch do_route(State, From, To, Packet) of
-	{'EXIT', Reason} ->
-	    Err = jlib:make_error_reply(Packet, ?ERR_INTERNAL_SERVER_ERROR),
-	    ejabberd_router:route(To, From, Err),
-	    %% Fail-Stop. Let the supervisor restarts us
-	    {stop, Reason, State};
+	Pid when is_pid(Pid) ->
+	    ok;
 	_ ->
-	    {noreply, State}
-    end;
+	    Err = jlib:make_error_reply(Packet, ?ERR_INTERNAL_SERVER_ERROR),
+	    ejabberd_router:route(To, From, Err)
+    end,
+    {noreply, State};
 
 handle_info(_Info, State) ->
     {noreply, State}.
@@ -208,65 +227,54 @@ process_local_iq(_From, _To, #iq{type = Type, lang = Lang, sub_el = SubEl} = IQ)
 			     [{xmlelement, "FN", [],
 			       [{xmlcdata, "ejabberd"}]},
 			      {xmlelement, "URL", [],
-			       [{xmlcdata,
-				 "http://ejabberd.jabber.ru/"}]},
+			       [{xmlcdata, ?EJABBERD_URI}]},
 			      {xmlelement, "DESC", [],
 			       [{xmlcdata,
 				 translate:translate(
 				   Lang,
-				   "Erlang Jabber Server\n"
-				   "Copyright (c) 2002-2006 Alexey Shchepin")}]},
+				   "Erlang Jabber Server") ++
+				   "\nCopyright (c) 2002-2008 ProcessOne"}]},
 			      {xmlelement, "BDAY", [],
 			       [{xmlcdata, "2002-11-16"}]}
 			     ]}]}
     end.
 
--define(SM_IQ_TIMEOUT, 20000).
-
-process_sm_iq(From, #jid{lserver=LServer} = To, #iq{sub_el = SubEl} = IQ) ->
-    Proc = gen_mod:get_module_proc(LServer, ?PROCNAME),
-    case catch gen_server:call(Proc,
-			       {process_sm_iq, From, To, IQ}, ?SM_IQ_TIMEOUT) of
-	{'EXIT', Reason} ->
-	    case Reason of
-		{timeout, _} ->
-		    IQ#iq{type = error,
-			  sub_el = [SubEl, ?ERR_REMOTE_SERVER_TIMEOUT]};
-		_ ->
-		    IQ#iq{type = error,
-			  sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]}
-	    end;
+process_sm_iq(_From, #jid{lserver=LServer} = To, #iq{sub_el = SubEl} = IQ) ->
+    case catch process_vcard_ldap(To, IQ, LServer) of
+	{'EXIT', _} ->
+	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_INTERNAL_SERVER_ERROR]};
 	Other ->
 	    Other
     end.
 
-handle_call({process_sm_iq, _From, To, IQ}, _FromPid, State) ->
+process_vcard_ldap(To, IQ, Server) ->
+    {ok, State} = eldap_utils:get_state(Server, ?PROCNAME),
     #iq{type = Type, sub_el = SubEl} = IQ,
-    Reply = case Type of
-		set ->
-		    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
-		get ->
-		    #jid{luser = LUser} = To,
-		    LServer = State#state.serverhost,
-		    case ejabberd_auth:is_user_exists(LUser, LServer) of
-			true ->
-			    VCardMap = State#state.vcard_map,
-			    case find_ldap_user(LUser, State) of
-				#eldap_entry{attributes = Attributes} ->
-				    Vcard = ldap_attributes_to_vcard(Attributes, VCardMap, {LUser, LServer}),
-				    IQ#iq{type = result, sub_el = Vcard};
-				_ ->
-				    IQ#iq{type = result, sub_el = []}
-			    end;
+    case Type of
+	set ->
+	    IQ#iq{type = error, sub_el = [SubEl, ?ERR_NOT_ALLOWED]};
+	get ->
+	    #jid{luser = LUser} = To,
+	    LServer = State#state.serverhost,
+	    case ejabberd_auth:is_user_exists(LUser, LServer) of
+		true ->
+		    VCardMap = State#state.vcard_map,
+		    case find_ldap_user(LUser, State) of
+			#eldap_entry{attributes = Attributes} ->
+			    Vcard = ldap_attributes_to_vcard(Attributes, VCardMap, {LUser, LServer}),
+			    IQ#iq{type = result, sub_el = Vcard};
 			_ ->
 			    IQ#iq{type = result, sub_el = []}
-		    end
-	    end,
-    {reply, Reply, State};
+		    end;
+		_ ->
+		    IQ#iq{type = result, sub_el = []}
+	    end
+	end.
 
+handle_call(get_state, _From, State) ->
+    {reply, {ok, State}, State};
 handle_call(stop, _From, State) ->
     {stop, normal, ok, State};
-
 handle_call(_Request, _From, State) ->
     {reply, bad_request, State}.
 
@@ -277,7 +285,7 @@ find_ldap_user(User, State) ->
     VCardAttrs = State#state.vcard_map_attrs,
     case eldap_filter:parse(RFC2254_Filter, [{"%u", User}]) of
 	{ok, EldapFilter} ->
-	    case eldap:search(Eldap_ID, [{base, Base},
+	    case eldap_pool:search(Eldap_ID, [{base, Base},
 					 {filter, EldapFilter},
 					 {attributes, VCardAttrs}]) of
 		#eldap_search_result{entries = [E | _]} ->
@@ -392,6 +400,9 @@ ldap_attribute_to_vcard(_, _) ->
 	  ] ++ lists:map(fun({X,Y}) -> ?TLFIELD("text-single", X, Y) end, SearchFields)}]).
 
 do_route(State, From, To, Packet) ->
+    spawn(?MODULE, route, [State, From, To, Packet]).
+
+route(State, From, To, Packet) ->
     #jid{user = User, resource = Resource} = To,
     if
 	(User /= "") or (Resource /= "") ->
@@ -447,7 +458,7 @@ do_route(State, From, To, Packet) ->
 						  From,
 						  jlib:iq_to_xml(ResIQ))
 		    end;
-		#iq{type = Type, xmlns = ?NS_DISCO_INFO} ->
+		#iq{type = Type, xmlns = ?NS_DISCO_INFO, lang = Lang} ->
 		    case Type of
 			set ->
 			    Err = jlib:make_error_reply(
@@ -463,7 +474,7 @@ do_route(State, From, To, Packet) ->
 						   [{"category", "directory"},
 						    {"type", "user"},
 						    {"name",
-						     "vCard User Search"}],
+						     translate:translate(Lang, "vCard User Search")}],
 						   []},
 						  {xmlelement, "feature",
 						   [{"var", ?NS_SEARCH}], []},
@@ -513,13 +524,12 @@ iq_get_vcard(Lang) ->
     [{xmlelement, "FN", [],
       [{xmlcdata, "ejabberd/mod_vcard"}]},
      {xmlelement, "URL", [],
-      [{xmlcdata,
-        "http://ejabberd.jabber.ru/"}]},
+      [{xmlcdata, ?EJABBERD_URI}]},
      {xmlelement, "DESC", [],
       [{xmlcdata, translate:translate(
 		    Lang,
-		    "ejabberd vCard module\n"
-		    "Copyright (c) 2003-2006 Alexey Shchepin")}]}].
+		    "ejabberd vCard module") ++
+		    "\nCopyright (c) 2003-2008 ProcessOne"}]}].
 
 -define(LFIELD(Label, Var),
 	{xmlelement, "field", [{"label", translate:translate(Lang, Label)},
@@ -531,9 +541,9 @@ search_result(Lang, JID, State, Data) ->
 	       [{xmlcdata, translate:translate(Lang, "Search Results for ") ++
 		 jlib:jid_to_string(JID)}]},
 	      {xmlelement, "reported", [],
-	       [?LFIELD("Jabber ID", "jid")] ++
+	       [?TLFIELD("text-single", "Jabber ID", "jid")] ++
 	       lists:map(
-		 fun({Name, Value}) -> ?LFIELD(Name, Value) end,
+		 fun({Name, Value}) -> ?TLFIELD("text-single", Name, Value) end,
 		 SearchReported)
 	      }],
     case search(State, Data) of
@@ -552,12 +562,13 @@ search(State, Data) ->
     Base = State#state.base,
     SearchFilter = State#state.search_filter,
     Eldap_ID = State#state.eldap_id,
-    UA = State#state.uid,
-    UAF = State#state.uid_format,
+    UIDs = State#state.uids,
+    Limit = State#state.matches,
     ReportedAttrs = State#state.search_reported_attrs,
-    Filter = eldap:'and'([SearchFilter, make_filter(Data, UA, UAF)]),
-    case eldap:search(Eldap_ID, [{base, Base},
+    Filter = eldap:'and'([SearchFilter, eldap_utils:make_filter(Data, UIDs)]),
+    case eldap_pool:search(Eldap_ID, [{base, Base},
 				 {filter, Filter},
+				 {limit, Limit},
 				 {attributes, ReportedAttrs}]) of
 	#eldap_search_result{entries = E} ->
 	    search_items(E, State);
@@ -569,8 +580,7 @@ search_items(Entries, State) ->
     LServer = State#state.serverhost,
     SearchReported = State#state.search_reported,
     VCardMap = State#state.vcard_map,
-    UIDAttr = State#state.uid,
-    UIDAttrFormat = State#state.uid_format,
+	UIDs = State#state.uids,
     Attributes = lists:map(
 		   fun(E) ->
 			   #eldap_entry{attributes = Attrs} = E,
@@ -578,12 +588,13 @@ search_items(Entries, State) ->
 		   end, Entries),
     lists:flatmap(
       fun(Attrs) ->
-	      U = get_ldap_attr(UIDAttr, Attrs),
-	      case get_user_part(U, UIDAttrFormat) of
-		  {ok, Username} ->
-		      case ejabberd_auth:is_user_exists(Username, LServer) of
-			  true ->
-			      RFields = lists:map(
+	      case eldap_utils:find_ldap_attrs(UIDs, Attrs) of
+	      {U, UIDAttrFormat} ->
+	          case eldap_utils:get_user_part(U, UIDAttrFormat) of
+		      {ok, Username} ->
+		          case ejabberd_auth:is_user_exists(Username, LServer) of
+			      true ->
+			        RFields = lists:map(
 					  fun({_, VCardName}) ->
 						  {VCardName,
 						   map_vcard_attr(
@@ -592,41 +603,20 @@ search_items(Entries, State) ->
 						     VCardMap,
 						     {Username, ?MYNAME})}
 					  end, SearchReported),
-			      Result = [?FIELD("jid", Username ++ "@" ++ LServer)] ++
-				  [?FIELD(Name, Value) || {Name, Value} <- RFields],
-			      [{xmlelement, "item", [], Result}];
-			  _ ->
-			      []
-		      end;
-		  _ ->
+			        Result = [?FIELD("jid", Username ++ "@" ++ LServer)] ++
+				    [?FIELD(Name, Value) || {Name, Value} <- RFields],
+			        [{xmlelement, "item", [], Result}];
+			      _ ->
+			          []
+		          end;
+		      _ ->
+		         []
+	          end;
+          "" ->
 		      []
-	      end
+		  end
       end, Attributes).
 
-make_filter(Data, UAttr, UAttrFormat) ->
-    Filter = lists:flatmap(
-	       fun({Name, [Value | _]}) ->
-		       case Name of
-			   "%u" when Value /= "" ->
-			       {ok, UAF, _} = regexp:sub(UAttrFormat, "%u", "*%u*"),
-			       case eldap_filter:parse(
-				      "("++UAttr++"="++UAF++")", [{"%u", Value}]) of
-				   {ok, F} -> [F];
-				   _ -> []
-			       end;
-			   _ when Value /= "" ->
-			       [eldap:substrings(Name, [{any, Value}])];
-			   _ ->
-			       []
-		       end
-	       end, Data),
-    case Filter of
-	[F] ->
-	    F;
-	_ ->
-	    eldap:'and'(Filter)
-    end.
-
 remove_user(_User) ->
     true.
 
@@ -634,39 +624,15 @@ remove_user(_User) ->
 %%% Auxiliary functions.
 %%%-----------------------
 
-get_user_part(String, Pattern) ->
-    F = fun(S, P) ->
-		First = string:str(P, "%u"),
-		TailLength = length(P) - (First+1),
-		string:sub_string(S, First, length(S) - TailLength)
-	end,
-    case catch F(String, Pattern) of
-	{'EXIT', _} ->
-	    {error, badmatch};
-	Result ->
-	    case regexp:sub(Pattern, "%u", Result) of
-		{ok, String, _} -> {ok, Result};
-		_ -> {error, badmatch}
-	    end
-    end.
-
-case_insensitive_match(X, Y) ->
-    X1 = stringprep:tolower(X),
-    Y1 = stringprep:tolower(Y),
-    if
-	X1 == Y1 -> true;
-	true -> false
-    end.
-
 map_vcard_attr(VCardName, Attributes, Pattern, UD) ->
     Res = lists:filter(
 	    fun({Name, _, _}) ->
-		    case_insensitive_match(Name, VCardName)
+		    eldap_utils:case_insensitive_match(Name, VCardName)
 	    end, Pattern),
     case Res of
 	[{_, Str, Attrs}] ->
 	    process_pattern(Str, UD,
-			    [get_ldap_attr(X, Attributes) || X<-Attrs]);
+			    [eldap_utils:get_ldap_attr(X, Attributes) || X<-Attrs]);
 	_ -> ""
     end.
 
@@ -674,16 +640,6 @@ process_pattern(Str, {User, Domain}, AttrValues) ->
 	eldap_filter:do_sub(Str,
 		[{"%s", V, 1} || V <- AttrValues] ++ [{"%u", User},{"%d", Domain}]).
 
-get_ldap_attr(LDAPAttr, Attributes) ->
-    Res = lists:filter(
-	    fun({Name, _}) ->
-		    case_insensitive_match(Name, LDAPAttr)
-	    end, Attributes),
-    case Res of
-	[{_, [Value|_]}] -> Value;
-	_ -> ""
-    end.
-
 find_xdata_el({xmlelement, _Name, _Attrs, SubEls}) ->
     find_xdata_el1(SubEls).
 
@@ -700,14 +656,23 @@ find_xdata_el1([_ | Els]) ->
     find_xdata_el1(Els).
 
 parse_options(Host, Opts) ->
-    MyHost = gen_mod:get_opt(host, Opts, "vjud." ++ Host),
+    MyHost = gen_mod:get_opt_host(Host, Opts, "vjud.@HOST@"),
     Search = gen_mod:get_opt(search, Opts, true),
+    Matches = case gen_mod:get_opt(matches, Opts, 30) of
+		infinity  -> 0;
+		N         -> N
+	    end,
     Eldap_ID = atom_to_list(gen_mod:get_module_proc(Host, ?PROCNAME)),
     LDAPServers = case gen_mod:get_opt(ldap_servers, Opts, undefined) of
 		      undefined ->
 			  ejabberd_config:get_local_option({ldap_servers, Host});
 		      S -> S
 		  end,
+    LDAPBackups = case gen_mod:get_opt(ldap_backups, Opts, undefined) of
+		      undefined ->
+			  ejabberd_config:get_local_option({ldap_servers, Host});
+		      Backups -> Backups
+		  end,
     LDAPPort = case gen_mod:get_opt(ldap_port, Opts, undefined) of
 		   undefined ->
 		       case ejabberd_config:get_local_option({ldap_port, Host}) of
@@ -721,22 +686,14 @@ parse_options(Host, Opts) ->
 		       ejabberd_config:get_local_option({ldap_base, Host});
 		   B -> B
 	       end,
-    UIDAttr = case gen_mod:get_opt(ldap_uidattr, Opts, undefined) of
-		  undefined ->
-		      case ejabberd_config:get_local_option({ldap_uidattr, Host}) of
-			  undefined -> "uid";
-			  UA -> UA
-		      end;
-		  UA -> UA
-	      end,
-    UIDAttrFormat = case gen_mod:get_opt(ldap_uidattr_format, Opts, undefined) of
-			undefined ->
-			    case ejabberd_config:get_local_option({ldap_uidattr_format, Host}) of
-				undefined -> "%u";
-				UAF -> UAF
-			    end;
-			UAF -> UAF
-		    end,
+	UIDs = case gen_mod:get_opt(ldap_uids, Opts, undefined) of
+	    undefined ->
+		    case ejabberd_config:get_local_option({ldap_uids, Host}) of
+			undefined -> [{"uid", "%u"}];
+			UI -> eldap_utils:uids_domain_subst(Host, UI)
+			end;
+		UI -> eldap_utils:uids_domain_subst(Host, UI)
+		end,
     RootDN = case gen_mod:get_opt(ldap_rootdn, Opts, undefined) of
 		 undefined ->
 		     case ejabberd_config:get_local_option({ldap_rootdn, Host}) of
@@ -753,7 +710,7 @@ parse_options(Host, Opts) ->
 		       end;
 		   Pass -> Pass
 	       end,
-    SubFilter = "("++UIDAttr++"="++UIDAttrFormat++")",
+    SubFilter = lists:flatten(eldap_utils:generate_subfilter(UIDs)),
     UserFilter = case gen_mod:get_opt(ldap_filter, Opts, undefined) of
 		     undefined ->
 			 case ejabberd_config:get_local_option({ldap_filter, Host}) of
@@ -772,8 +729,9 @@ parse_options(Host, Opts) ->
     %% In search requests we need to fetch only attributes defined
     %% in vcard-map and search-reported. In some cases,
     %% this will essentially reduce network traffic from an LDAP server.
+	UIDAttrs = [UAttr || {UAttr, _} <- UIDs],
     VCardMapAttrs = lists:usort(
-		      lists:append([A || {_, _, A} <- VCardMap]) ++ [UIDAttr]),
+		      lists:append([A || {_, _, A} <- VCardMap]) ++ UIDAttrs),
     SearchReportedAttrs =
 	lists:usort(lists:flatmap(
 		      fun({_, N}) ->
@@ -781,23 +739,24 @@ parse_options(Host, Opts) ->
 				  {value, {_, _, L}} -> L;
 				  _ -> []
 			      end
-		      end, SearchReported) ++ [UIDAttr]),
+		      end, SearchReported) ++ UIDAttrs),
     #state{serverhost = Host,
 	   myhost = MyHost,
 	   eldap_id = Eldap_ID,
 	   search = Search,
 	   servers = LDAPServers,
+	   backups = LDAPBackups,
 	   port = LDAPPort,
 	   dn = RootDN,
 	   base = LDAPBase,
 	   password = Password,
-	   uid = UIDAttr,
-	   uid_format = UIDAttrFormat,
+	   uids = UIDs,
 	   vcard_map = VCardMap,
 	   vcard_map_attrs = VCardMapAttrs,
 	   user_filter = UserFilter,
 	   search_filter = SearchFilter,
 	   search_fields = SearchFields,
 	   search_reported = SearchReported,
-	   search_reported_attrs = SearchReportedAttrs
+	   search_reported_attrs = SearchReportedAttrs,
+	   matches = Matches
 	  }.
