@@ -1,3 +1,23 @@
+/*
+ * ejabberd, Copyright (C) 2002-2008   Process-one
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *                         
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ * 02111-1307 USA
+ *
+ */
+
 SET ANSI_NULLS ON
 GO
 SET QUOTED_IDENTIFIER ON
@@ -84,10 +104,14 @@ if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[vcard]') a
 drop table [dbo].[vcard]
 GO
 
+if exists (select * from dbo.sysobjects where id = object_id(N'[dbo].[private_storage]') and OBJECTPROPERTY(id, N'IsUserTable') = 1)
+drop table [dbo].[private_storage]
+GO
+
 CREATE TABLE [dbo].[last] (
 	[username] [varchar] (250) NOT NULL ,
 	[seconds] [varchar] (50) NOT NULL ,
-	[state] [varchar] (100) NULL ,
+	[state] [varchar] (100) NOT NULL ,
 ) ON [PRIMARY]
 GO
 
@@ -101,10 +125,10 @@ GO
 CREATE TABLE [dbo].[rosterusers] (
 	[username] [varchar] (250) NOT NULL ,
 	[jid] [varchar] (250) NOT NULL ,
-	[nick] [varchar] (50) NULL ,
+	[nick] [varchar] (50) NOT NULL ,
 	[subscription] [char] (1) NOT NULL ,
 	[ask] [char] (1) NOT NULL ,
-	[askmessage] [varchar] (250) NULL ,
+	[askmessage] [varchar] (250) NOT NULL ,
 	[server] [char] (1) NOT NULL ,
 	[subscribe] [varchar] (200) NULL ,
 	[type] [varchar] (50) NULL 
@@ -114,7 +138,7 @@ GO
 CREATE TABLE [dbo].[spool] (
 	[id] [numeric](19, 0) IDENTITY (1, 1) NOT NULL ,
 	[username] [varchar] (250) NOT NULL ,
-	[xml] [text] NULL ,
+	[xml] [text] NOT NULL ,
 	[notifyprocessed] [bit] NULL ,
 	[created] [datetime] NULL ,
 ) ON [PRIMARY] TEXTIMAGE_ON [PRIMARY]
@@ -151,6 +175,44 @@ CREATE TABLE [dbo].[vcard] (
 ) ON [PRIMARY]
 GO
 
+CREATE TABLE [dbo].[private_storage] (
+    [username] [varchar] (250) NOT NULL ,
+    [namespace] [varchar] (250) NOT NULL ,
+    [data] [text] NOT NULL
+) ON [PRIMARY]
+GO
+
+CREATE TABLE [dbo].[privacy_default_list] (
+    [username] [varchar] (250) NOT NULL,
+    [name] [varchar] (250) NOT NULL
+) ON [PRIMARY]
+GO
+
+CREATE TABLE [dbo].[privacy_list] (
+    [username] [varchar] (250) NOT NULL,
+    [name] [varchar] (250) NOT NULL,
+    id BIGINT UNSIGNED NOT NULL AUTO_INCREMENT UNIQUE
+) ON [PRIMARY]
+GO
+
+CREATE TABLE [dbo].[privacy_list_data] (
+    [id] [bigint] NOT NULL,
+    [t] [character] (1) NOT NULL,
+    [value] [text] NOT NULL,
+    [action] [character] (1) NOT NULL,
+    [ord] [NUMERIC] NOT NULL,
+    [match_all] [boolean] NOT NULL,
+    [match_iq] [boolean] NOT NULL,
+    [match_message] [boolean] NOT NULL,
+    [match_presence_in] [boolean] NOT NULL,
+    [match_presence_out] [boolean] NOT NULL
+) ON [PRIMARY]
+GO
+
+/* Constraints to add:
+- id in privacy_list is a SERIAL autogenerated number
+- id in privacy_list_data must exist in the table privacy_list */
+
 ALTER TABLE [dbo].[last] WITH NOCHECK ADD 
 	CONSTRAINT [PK_last] PRIMARY KEY  CLUSTERED 
 	(
@@ -189,7 +251,7 @@ ALTER TABLE [dbo].[vcard] WITH NOCHECK ADD
 	) WITH  FILLFACTOR = 90  ON [PRIMARY] 
 GO
 
- CREATE  CLUSTERED  INDEX [IX_rosterusers_user] ON [dbo].[rosterusers]([username]) WITH  FILLFACTOR = 90 ON [PRIMARY]
+CREATE  CLUSTERED  INDEX [IX_rosterusers_user] ON [dbo].[rosterusers]([username]) WITH  FILLFACTOR = 90 ON [PRIMARY]
 GO
 
 ALTER TABLE [dbo].[last] WITH NOCHECK ADD 
@@ -206,6 +268,13 @@ ALTER TABLE [dbo].[users] WITH NOCHECK ADD
 	CONSTRAINT [DF_users_created] DEFAULT (getdate()) FOR [created]
 GO
 
+ALTER TABLE [dbo].[privacy_default_list] WITH NOCHECK ADD 
+	CONSTRAINT [PK_privacy_defaut_list] PRIMARY KEY  CLUSTERED 
+	(
+		[username]
+	) WITH  FILLFACTOR = 90  ON [PRIMARY] 
+GO
+
  CREATE  INDEX [IX_rostergroups_jid] ON [dbo].[rostergroups]([jid]) WITH  FILLFACTOR = 90 ON [PRIMARY]
 GO
 
@@ -227,16 +296,28 @@ GO
  CREATE  INDEX [IK_Spool_Created] ON [dbo].[spool]([created]) WITH  FILLFACTOR = 90 ON [PRIMARY]
 GO
 
+ CREATE  INDEX [IX_private_user] ON [dbo].[private_storage]([username]) WITH  FILLFACTOR = 90 ON [PRIMARY]
+GO
+
+ CREATE  INDEX [IX_private_user_ns] ON [dbo].[private_storage]([username], [namespace]) WITH  FILLFACTOR = 90 ON [PRIMARY]
+GO
+
+ CREATE INDEX [IX_privacy_list_username] ON [dbo].[privacy_list]([username]) WITH  FILLFACTOR = 90 ON [PRIMARY]
+GO
+
+ CREATE INDEX [IX_privacy_list_username_name] ON [dbo].[privacy_list]([username], [name]) WITH  FILLFACTOR = 90 ON [PRIMARY]
+GO
+
 /*********************************************************/
 /** These store procedures are for use with ejabberd    **/
 /** 1.1 and Microsoft Sql Server 2000                   **/
 /**                                                     **/
 /** The stored procedures reduce the need to sql        **/
 /** compilation of the database and also allow for also **/
-/** provide each of database integration. The store     **/
+/** provide each of database integration. The stored    **/
 /** procedure have been optimized to increase database  **/
 /** performance and a reduction of 80% in CPU was       **/
-/** achive over the use of standard sql.                **/
+/** achieved over the use of standard sql.              **/
 /*********************************************************/
 
 /****** Object:  StoredProcedure [dbo].[add_roster] ******/
@@ -455,7 +536,7 @@ CREATE PROCEDURE [dbo].[add_user]
 AS
 BEGIN
   INSERT INTO users 
-    ( username, 
+    ( [username], 
       [password]
     ) 
   VALUES 
@@ -484,6 +565,7 @@ BEGIN
     END
 END
 GO
+
 /******************************************************************/
 /****** Object:  StoredProcedure [dbo].[get_password]            **/
 /** Retrive the user password                                    **/
@@ -557,7 +639,6 @@ BEGIN
     DELETE FROM rostergroups 
     WITH (ROWLOCK)
     WHERE (rostergroups.username = @Username) AND (rostergroups.jid = @JID);
-
   COMMIT
 END
 GO
@@ -627,7 +708,6 @@ BEGIN
     DELETE FROM rostergroups 
     WITH (ROWLOCK)
     WHERE rostergroups.username = @Username;
-
   COMMIT
 END
 GO
@@ -682,7 +762,7 @@ GO
 /** Retrive the user roster                                      **/
 /******************************************************************/
 CREATE PROCEDURE [dbo].[get_roster]
-  @Username varchar(200)
+  @Username varchar(250)
 AS
 DECLARE
   @vRosterusers table( username      varchar(1), 
@@ -855,7 +935,7 @@ GO
 
 /******************************************************************/
 /****** Object:  StoredProcedure [dbo].[list_users]              **/
-/** Retrive a list of all users                                  **/
+/** Retrieve a list of all users                                 **/
 /******************************************************************/
 CREATE PROCEDURE [dbo].[list_users]
 AS
@@ -897,3 +977,69 @@ BEGIN
     END
 END
 GO
+
+/******************************************************************/
+/****** Object:  StoredProcedure [dbo].[set_private_data]        **/
+/** store user private data by namespace                         **/
+/******************************************************************/
+CREATE PROCEDURE [dbo].[set_private_data]
+    @Username  varchar(250),
+    @Namespace varchar(250),
+    @Data varchar(8000)
+AS
+BEGIN
+  IF EXISTS (SELECT username FROM private_storage with (nolock) WHERE private_storage.username = @Username AND private_storage.namespace = @Namespace)
+    BEGIN
+        UPDATE [private_storage]
+        SET [private_storage].username = @Username,
+            [private_storage].namespace = @Namespace,
+            [private_storage].data = @Data
+            WHERE private_storage.username = @Username AND private_storage.namespace = @Namespace;
+    END
+  ELSE
+    BEGIN
+        INSERT INTO [private_storage]
+            ( [private_storage].username,
+              [private_storage].namespace,
+              [private_storage].data
+            )
+        VALUES
+            ( @Username,
+              @Namespace,
+              @Data
+            )
+    END
+END
+GO
+
+/******************************************************************/
+/****** Object:  StoredProcedure [dbo].[get_private_data]        **/
+/** Retrieve user private data by namespace                      **/
+/******************************************************************/
+CREATE PROCEDURE [dbo].[get_private_data]
+    @Username  varchar(250),
+    @Namespace varchar(250)
+AS
+BEGIN
+  SELECT private_storage.data AS data
+  FROM private_storage WITH (NOLOCK)
+  WHERE username=@Username and namespace=@Namespace;
+END
+GO
+
+/***************************************************************/
+/****** Object:  StoredProcedure [dbo].[del_user_storage] ******/
+/** Delete private storage area for a given user              **/
+/***************************************************************/
+CREATE PROCEDURE [dbo].[del_user_storage]
+  @Username  varchar(250)
+AS
+BEGIN
+  DELETE FROM [private_storage]
+  WITH (ROWLOCK)
+  WHERE [private_storage].username=@Username;
+END
+GO
+
+
+
