@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : tls.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Interface to openssl
-%%% Created : 24 Jul 2004 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: tls.erl 483 2006-01-13 01:55:20Z alexey $
+%%% Created : 24 Jul 2004 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(tls).
--author('alexey@sevcom.net').
--vsn('$Revision: 483 $ ').
+-author('alexey@process-one.net').
 
 -behaviour(gen_server).
 
@@ -17,6 +34,7 @@
 	 send/2,
 	 recv/2, recv/3, recv_data/2,
 	 setopts/2,
+	 sockname/1, peername/1,
 	 controlling_process/2,
 	 close/1,
 	 get_peer_certificate/1,
@@ -31,6 +49,8 @@
 	 code_change/3,
 	 terminate/2]).
 
+-include("ejabberd.hrl").
+
 -define(SET_CERTIFICATE_FILE_ACCEPT, 1).
 -define(SET_CERTIFICATE_FILE_CONNECT, 2).
 -define(SET_ENCRYPTED_INPUT,  3).
@@ -140,7 +160,7 @@ recv_data(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 			<<0, Out/binary>> ->
 			    case gen_tcp:send(TCPSocket, Out) of
 				ok ->
-				    %io:format("IN: ~p~n", [{TCPSocket, binary_to_list(In)}]),
+				    %?PRINT("IN: ~p~n", [{TCPSocket, binary_to_list(In)}]),
 				    {ok, In};
 				Error ->
 				    Error
@@ -158,7 +178,7 @@ recv_data(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 send(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
     case port_control(Port, ?SET_DECRYPTED_OUTPUT, Packet) of
 	<<0>> ->
-	    %io:format("OUT: ~p~n", [{TCPSocket, lists:flatten(Packet)}]),
+	    %?PRINT("OUT: ~p~n", [{TCPSocket, lists:flatten(Packet)}]),
 	    case port_control(Port, ?GET_ENCRYPTED_OUTPUT, []) of
 		<<0, Out/binary>> ->
 		    gen_tcp:send(TCPSocket, Out);
@@ -180,6 +200,12 @@ send(#tlssock{tcpsock = TCPSocket, tlsport = Port}, Packet) ->
 setopts(#tlssock{tcpsock = TCPSocket}, Opts) ->
     inet:setopts(TCPSocket, Opts).
 
+sockname(#tlssock{tcpsock = TCPSocket}) ->
+    inet:sockname(TCPSocket).
+
+peername(#tlssock{tcpsock = TCPSocket}) ->
+    inet:peername(TCPSocket).
+
 controlling_process(#tlssock{tcpsock = TCPSocket}, Pid) ->
     gen_tcp:controlling_process(TCPSocket, Pid).
 
@@ -211,50 +237,50 @@ test() ->
 	{error, already_loaded} -> ok
     end,
     Port = open_port({spawn, tls_drv}, [binary]),
-    io:format("open_port: ~p~n", [Port]),
+    ?PRINT("open_port: ~p~n", [Port]),
     PCRes = port_control(Port, ?SET_CERTIFICATE_FILE_ACCEPT,
 			 "./ssl.pem" ++ [0]),
-    io:format("port_control: ~p~n", [PCRes]),
+    ?PRINT("port_control: ~p~n", [PCRes]),
     {ok, ListenSocket} = gen_tcp:listen(1234, [binary,
 					       {packet, 0}, 
 					       {active, true},
 					       {reuseaddr, true},
 					       {nodelay, true}]),
-    io:format("listen: ~p~n", [ListenSocket]),
+    ?PRINT("listen: ~p~n", [ListenSocket]),
     {ok, Socket} = gen_tcp:accept(ListenSocket),
-    io:format("accept: ~p~n", [Socket]),
+    ?PRINT("accept: ~p~n", [Socket]),
     loop(Port, Socket).
 
 
 loop(Port, Socket) ->
     receive
 	{tcp, Socket, Data} ->
-	    %io:format("read: ~p~n", [Data]),
+	    %?PRINT("read: ~p~n", [Data]),
 	    Res = port_control(Port, ?SET_ENCRYPTED_INPUT, Data),
-	    io:format("SET_ENCRYPTED_INPUT: ~p~n", [Res]),
+	    ?PRINT("SET_ENCRYPTED_INPUT: ~p~n", [Res]),
 
 	    DIRes = port_control(Port, ?GET_DECRYPTED_INPUT, Data),
-	    io:format("GET_DECRYPTED_INPUT: ~p~n", [DIRes]),
+	    ?PRINT("GET_DECRYPTED_INPUT: ~p~n", [DIRes]),
 	    case DIRes of
 		<<0, In/binary>> ->
-		    io:format("input: ~s~n", [binary_to_list(In)]);
+		    ?PRINT("input: ~s~n", [binary_to_list(In)]);
 		<<1, DIError/binary>> ->
-		    io:format("GET_DECRYPTED_INPUT error: ~p~n", [binary_to_list(DIError)])
+		    ?PRINT("GET_DECRYPTED_INPUT error: ~p~n", [binary_to_list(DIError)])
 	    end,
 
 	    EORes = port_control(Port, ?GET_ENCRYPTED_OUTPUT, Data),
-	    io:format("GET_ENCRYPTED_OUTPUT: ~p~n", [EORes]),
+	    ?PRINT("GET_ENCRYPTED_OUTPUT: ~p~n", [EORes]),
 	    case EORes of
 		<<0, Out/binary>> ->
 		    gen_tcp:send(Socket, Out);
 		<<1, EOError/binary>> ->
-		    io:format("GET_ENCRYPTED_OUTPUT error: ~p~n", [binary_to_list(EOError)])
+		    ?PRINT("GET_ENCRYPTED_OUTPUT error: ~p~n", [binary_to_list(EOError)])
 	    end,
 		    
 
 	    loop(Port, Socket);
 	Msg ->
-	    io:format("receive: ~p~n", [Msg]),
+	    ?PRINT("receive: ~p~n", [Msg]),
 	    loop(Port, Socket)
     end.
 
