@@ -1,4 +1,22 @@
-/* $Id: tls_drv.c 593 2006-08-04 02:25:10Z alexey $ */
+/*
+ * ejabberd, Copyright (C) 2002-2008   Process-one
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
+ * published by the Free Software Foundation; either version 2 of the
+ * License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+ * General Public License for more details.
+ *                         
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software
+ * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+ * 02111-1307 USA
+ *
+ */
 
 #include <stdio.h>
 #include <string.h>
@@ -108,7 +126,7 @@ static int tls_drv_control(ErlDrvData handle,
 	 d->ctx = SSL_CTX_new(SSLv23_method());
 	 die_unless(d->ctx, "SSL_CTX_new failed");
 
-	 res = SSL_CTX_use_certificate_file(d->ctx, buf, SSL_FILETYPE_PEM);
+	 res = SSL_CTX_use_certificate_chain_file(d->ctx, buf);
 	 die_unless(res > 0, "SSL_CTX_use_certificate_file failed");
 
 	 res = SSL_CTX_use_PrivateKey_file(d->ctx, buf, SSL_FILETYPE_PEM);
@@ -232,7 +250,7 @@ static int tls_drv_control(ErlDrvData handle,
 	       rlen++;
 	       b = driver_alloc_binary(rlen);
 	       b->orig_bytes[0] = 0;
-	       tmp_buf = &b->orig_bytes[1];
+	       tmp_buf = (unsigned char *)&b->orig_bytes[1];
 	       i2d_X509(cert, &tmp_buf);
 	       X509_free(cert);
 	       *rbuf = (char *)b;
