@@ -1,13 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : translate.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
-%%% Purpose : 
-%%% Created :  6 Jan 2003 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: translate.erl 271 2004-09-30 21:54:39Z aleksey $
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
+%%% Purpose : Localization helper
+%%% Created :  6 Jan 2003 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(translate).
--author('alexey@sevcom.net').
+-author('alexey@process-one.net').
 
 -export([start/0,
 	 load_dir/1,
