@@ -1,14 +1,31 @@
 %%%----------------------------------------------------------------------
 %%% File    : xml_stream.erl
-%%% Author  : Alexey Shchepin <alexey@sevcom.net>
+%%% Author  : Alexey Shchepin <alexey@process-one.net>
 %%% Purpose : Parse XML streams
-%%% Created : 17 Nov 2002 by Alexey Shchepin <alexey@sevcom.net>
-%%% Id      : $Id: xml_stream.erl 537 2006-04-22 03:35:13Z alexey $
+%%% Created : 17 Nov 2002 by Alexey Shchepin <alexey@process-one.net>
+%%%
+%%%
+%%% ejabberd, Copyright (C) 2002-2008   Process-one
+%%%
+%%% This program is free software; you can redistribute it and/or
+%%% modify it under the terms of the GNU General Public License as
+%%% published by the Free Software Foundation; either version 2 of the
+%%% License, or (at your option) any later version.
+%%%
+%%% This program is distributed in the hope that it will be useful,
+%%% but WITHOUT ANY WARRANTY; without even the implied warranty of
+%%% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
+%%% General Public License for more details.
+%%%                         
+%%% You should have received a copy of the GNU General Public License
+%%% along with this program; if not, write to the Free Software
+%%% Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA
+%%% 02111-1307 USA
+%%%
 %%%----------------------------------------------------------------------
 
 -module(xml_stream).
--author('alexey@sevcom.net').
--vsn('$Revision: 537 $ ').
+-author('alexey@process-one.net').
 
 -export([new/1,
 	 new/2,
@@ -59,6 +76,15 @@ process_data(CallbackPid, Stack, Data) ->
 	    case Stack of
 		[El] ->
 		    [El];
+		%% Merge CDATA nodes if they are contiguous
+		%% This does not change the semantic: the split in
+		%% several CDATA nodes depends on the TCP/IP packet
+		%% fragmentation
+		[{xmlelement, Name, Attrs,
+		  [{xmlcdata, PreviousCData}|Els]} | Tail] ->
+		    [{xmlelement, Name, Attrs,
+		      [{xmlcdata, concat_binary([PreviousCData, CData])} | Els]} | Tail];
+		%% No previous CDATA
 		[{xmlelement, Name, Attrs, Els} | Tail] ->
 		    [{xmlelement, Name, Attrs, [{xmlcdata, CData} | Els]} |
 		     Tail];
